# -*- coding: utf-8 -*-
# ********************************************************************
#
#  $Id: svn_id $
#
#  High-level API for YCarbonDioxide
#
#  - - - - - - - - - License information: - - - - - - - - -
#
#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#
#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#  non-exclusive license to use, modify, copy and integrate this
#  file into your software for the sole purpose of interfacing
#  with Yoctopuce products.
#
#  You may reproduce and distribute copies of this file in
#  source or object form, as long as the sole purpose of this
#  code is to interface with Yoctopuce products. You must retain
#  this notice in the distributed source file.
#
#  You should refer to Yoctopuce General Terms and Conditions
#  for additional information regarding your rights and
#  obligations.
#
#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#  WARRANTY, OR OTHERWISE.
#
# *********************************************************************
"""
Yoctopuce library: High-level API for YCarbonDioxide
version: PATCH_WITH_VERSION
requires: yocto_carbondioxide_aio
requires: yocto_api
"""
from __future__ import annotations

import sys

# On MicroPython, code below will be wiped out at compile time
if sys.implementation.name != "micropython":
    # In CPython, enable edit-time type checking, including Final declaration
    from typing import Union, Final
    from collections.abc import Callable, Awaitable
    from .yocto_api import _IS_MICROPYTHON, _DYNAMIC_HELPERS
else:
    # In our micropython VM, common generic types are global built-ins
    # Others such as TypeVar should be avoided when using micropython,
    # as they produce overhead in runtime code
    # Final is translated into const() expressions before compilation
    _IS_MICROPYTHON: Final[bool] = True # noqa
    _DYNAMIC_HELPERS: Final[bool] = True # noqa

from .yocto_carbondioxide_aio import YCarbonDioxide as YCarbonDioxide_aio
from .yocto_api import (
    YAPIContext, YAPI, YSensor, YMeasure
)

# --- (YCarbonDioxide class start)
if not _IS_MICROPYTHON:
    # For CPython, use strongly typed callback types
    try:
        YCarbonDioxideValueCallback = Union[Callable[['YCarbonDioxide', str], Awaitable[None]], None]
        YCarbonDioxideTimedReportCallback = Union[Callable[['YCarbonDioxide', YMeasure], Awaitable[None]], None]
    except TypeError:
        YCarbonDioxideValueCallback = Union[Callable, Awaitable]
        YCarbonDioxideTimedReportCallback = Union[Callable, Awaitable]

# noinspection PyProtectedMember
class YCarbonDioxide(YSensor):
    """
    The YCarbonDioxide class allows you to read and configure Yoctopuce CO2 sensors.
    It inherits from YSensor class the core functions to read measures,
    to register callback functions, and to access the autonomous datalogger.
    This class adds the ability to perform manual calibration if required.

    """
    _aio: YCarbonDioxide_aio
    # --- (end of YCarbonDioxide class start)
    if not _IS_MICROPYTHON:
        # --- (YCarbonDioxide return codes)
        ABCPERIOD_INVALID: Final[int] = YAPI.INVALID_UINT
        COMMAND_INVALID: Final[str] = YAPI.INVALID_STRING
        # --- (end of YCarbonDioxide return codes)


    # --- (YCarbonDioxide implementation)

    @classmethod
    def FirstCarbonDioxide(cls) -> Union[YCarbonDioxide, None]:
        """
        Starts the enumeration of CO2 sensors currently accessible.
        Use the method YCarbonDioxide.nextCarbonDioxide() to iterate on
        next CO2 sensors.

        @return a pointer to a YCarbonDioxide object, corresponding to
                the first CO2 sensor currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YCarbonDioxide_aio.FirstCarbonDioxide())

    @classmethod
    def FirstCarbonDioxideInContext(cls, yctx: YAPIContext) -> Union[YCarbonDioxide, None]:
        """
        Starts the enumeration of CO2 sensors currently accessible.
        Use the method YCarbonDioxide.nextCarbonDioxide() to iterate on
        next CO2 sensors.

        @param yctx : a YAPI context.

        @return a pointer to a YCarbonDioxide object, corresponding to
                the first CO2 sensor currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YCarbonDioxide_aio.FirstCarbonDioxideInContext(yctx))

    def nextCarbonDioxide(self):
        """
        Continues the enumeration of CO2 sensors started using yFirstCarbonDioxide().
        Caution: You can't make any assumption about the returned CO2 sensors order.
        If you want to find a specific a CO2 sensor, use CarbonDioxide.findCarbonDioxide()
        and a hardwareID or a logical name.

        @return a pointer to a YCarbonDioxide object, corresponding to
                a CO2 sensor currently online, or a None pointer
                if there are no more CO2 sensors to enumerate.
        """
        return self._proxy(type(self), self._aio.nextCarbonDioxide())

    if not _DYNAMIC_HELPERS:
        def get_abcPeriod(self) -> int:
            """
            Returns the Automatic Baseline Calibration period, in hours. A negative value
            means that automatic baseline calibration is disabled.

            @return an integer corresponding to the Automatic Baseline Calibration period, in hours

            On failure, throws an exception or returns YCarbonDioxide.ABCPERIOD_INVALID.
            """
            return self._run(self._aio.get_abcPeriod())

    if not _DYNAMIC_HELPERS:
        def set_abcPeriod(self, newval: int) -> int:
            """
            Changes Automatic Baseline Calibration period, in hours. If you need
            to disable automatic baseline calibration (for instance when using the
            sensor in an environment that is constantly above 400 ppm CO2), set the
            period to -1. For the Yocto-CO2-V2, the only possible values are 24 and -1.
            Remember to call the saveToFlash() method of the
            module if the modification must be kept.

            @param newval : an integer corresponding to Automatic Baseline Calibration period, in hours

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_abcPeriod(newval))

    if not _DYNAMIC_HELPERS:
        def set_command(self, newval: str) -> int:
            return self._run(self._aio.set_command(newval))

    @classmethod
    def FindCarbonDioxide(cls, func: str) -> YCarbonDioxide:
        """
        Retrieves a CO2 sensor for a given identifier.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the CO2 sensor is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YCarbonDioxide.isOnline() to test if the CO2 sensor is
        indeed online at a given time. In case of ambiguity when looking for
        a CO2 sensor by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        If a call to this object's is_online() method returns FALSE although
        you are certain that the matching device is plugged, make sure that you did
        call registerHub() at application initialization time.

        @param func : a string that uniquely characterizes the CO2 sensor, for instance
                YCO2MK02.carbonDioxide.

        @return a YCarbonDioxide object allowing you to drive the CO2 sensor.
        """
        return cls._proxy(cls, YCarbonDioxide_aio.FindCarbonDioxide(func))

    @classmethod
    def FindCarbonDioxideInContext(cls, yctx: YAPIContext, func: str) -> YCarbonDioxide:
        """
        Retrieves a CO2 sensor for a given identifier in a YAPI context.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the CO2 sensor is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YCarbonDioxide.isOnline() to test if the CO2 sensor is
        indeed online at a given time. In case of ambiguity when looking for
        a CO2 sensor by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        @param yctx : a YAPI context
        @param func : a string that uniquely characterizes the CO2 sensor, for instance
                YCO2MK02.carbonDioxide.

        @return a YCarbonDioxide object allowing you to drive the CO2 sensor.
        """
        return cls._proxy(cls, YCarbonDioxide_aio.FindCarbonDioxideInContext(yctx, func))

    if not _IS_MICROPYTHON:
        def registerValueCallback(self, callback: YCarbonDioxideValueCallback) -> int:
            """
            Registers the callback function that is invoked on every change of advertised value.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and the character string describing
                    the new advertised value.
            @noreturn
            """
            return super().registerValueCallback(callback)

    if not _IS_MICROPYTHON:
        def registerTimedReportCallback(self, callback: YCarbonDioxideTimedReportCallback) -> int:
            """
            Registers the callback function that is invoked on every periodic timed notification.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and an YMeasure object describing
                    the new advertised value.
            @noreturn
            """
            return super().registerTimedReportCallback(callback)

    if not _DYNAMIC_HELPERS:
        def triggerForcedCalibration(self, refVal: float) -> int:
            """
            Triggers a forced calibration of the sensor at a given CO2 level, specified
            between 400ppm and 2000ppm. Before invoking this command, the sensor must
            have been maintained within the specified CO2 density during at least two
            minutes.

            @param refVal : reference CO2 density for the calibration

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.triggerForcedCalibration(refVal))

    if not _DYNAMIC_HELPERS:
        def triggerBaselineCalibration(self) -> int:
            """
            Triggers a baseline calibration at standard CO2 ambiant level (400ppm).
            It is normally not necessary to manually calibrate the sensor, because
            the built-in automatic baseline calibration procedure will automatically
            fix any long-term drift based on the lowest level of CO2 observed over the
            automatic calibration period. However, if automatic baseline calibration
            is disabled, you may want to manually trigger a calibration from time to
            time. Before starting a baseline calibration, make sure to put the sensor
            in a standard environment (e.g. outside in fresh air) at around 400 ppm
            for at least two minutes.

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.triggerBaselineCalibration())

    if not _DYNAMIC_HELPERS:
        def triggerZeroCalibration(self) -> int:
            """
            Triggers a zero calibration of the sensor on carbon dioxide-free air -
            for use with first generation Yocto-CO2 only.
            It is normally not necessary to manually calibrate the sensor, because
            the built-in automatic baseline calibration procedure will automatically
            fix any long-term drift based on the lowest level of CO2 observed over the
            automatic calibration period. However, if you disable automatic baseline
            calibration, you may want to manually trigger a calibration from time to
            time. Before starting a zero calibration, you should circulate carbon
            dioxide-free air within the sensor for a minute or two, using a small pipe
            connected to the sensor. Please contact support@yoctopuce.com for more details
            on the zero calibration procedure.

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.triggerZeroCalibration())

    # --- (end of YCarbonDioxide implementation)




# -*- coding: utf-8 -*-
# ********************************************************************
#
#  $Id: svn_id $
#
#  High-level API for YMagnetometer
#
#  - - - - - - - - - License information: - - - - - - - - -
#
#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#
#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#  non-exclusive license to use, modify, copy and integrate this
#  file into your software for the sole purpose of interfacing
#  with Yoctopuce products.
#
#  You may reproduce and distribute copies of this file in
#  source or object form, as long as the sole purpose of this
#  code is to interface with Yoctopuce products. You must retain
#  this notice in the distributed source file.
#
#  You should refer to Yoctopuce General Terms and Conditions
#  for additional information regarding your rights and
#  obligations.
#
#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#  WARRANTY, OR OTHERWISE.
#
# *********************************************************************
"""
Yoctopuce library: High-level API for YMagnetometer
version: PATCH_WITH_VERSION
requires: yocto_magnetometer_aio
requires: yocto_api
"""
from __future__ import annotations

import sys

# On MicroPython, code below will be wiped out at compile time
if sys.implementation.name != "micropython":
    # In CPython, enable edit-time type checking, including Final declaration
    from typing import Union, Final
    from collections.abc import Callable, Awaitable
    from .yocto_api import _IS_MICROPYTHON, _DYNAMIC_HELPERS
else:
    # In our micropython VM, common generic types are global built-ins
    # Others such as TypeVar should be avoided when using micropython,
    # as they produce overhead in runtime code
    # Final is translated into const() expressions before compilation
    _IS_MICROPYTHON: Final[bool] = True # noqa
    _DYNAMIC_HELPERS: Final[bool] = True # noqa

from .yocto_magnetometer_aio import YMagnetometer as YMagnetometer_aio
from .yocto_api import (
    YAPIContext, YAPI, YSensor, YMeasure
)

# --- (YMagnetometer class start)
if not _IS_MICROPYTHON:
    # For CPython, use strongly typed callback types
    try:
        YMagnetometerValueCallback = Union[Callable[['YMagnetometer', str], Awaitable[None]], None]
        YMagnetometerTimedReportCallback = Union[Callable[['YMagnetometer', YMeasure], Awaitable[None]], None]
    except TypeError:
        YMagnetometerValueCallback = Union[Callable, Awaitable]
        YMagnetometerTimedReportCallback = Union[Callable, Awaitable]

# noinspection PyProtectedMember
class YMagnetometer(YSensor):
    """
    The YSensor class is the parent class for all Yoctopuce sensor types. It can be
    used to read the current value and unit of any sensor, read the min/max
    value, configure autonomous recording frequency and access recorded data.
    It also provides a function to register a callback invoked each time the
    observed value changes, or at a predefined interval. Using this class rather
    than a specific subclass makes it possible to create generic applications
    that work with any Yoctopuce sensor, even those that do not yet exist.
    Note: The YAnButton class is the only analog input which does not inherit
    from YSensor.

    """
    _aio: YMagnetometer_aio
    # --- (end of YMagnetometer class start)
    if not _IS_MICROPYTHON:
        # --- (YMagnetometer return codes)
        BANDWIDTH_INVALID: Final[int] = YAPI.INVALID_UINT
        XVALUE_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        YVALUE_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        ZVALUE_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        # --- (end of YMagnetometer return codes)


    # --- (YMagnetometer implementation)

    @classmethod
    def FirstMagnetometer(cls) -> Union[YMagnetometer, None]:
        """
        Starts the enumeration of magnetometers currently accessible.
        Use the method YMagnetometer.nextMagnetometer() to iterate on
        next magnetometers.

        @return a pointer to a YMagnetometer object, corresponding to
                the first magnetometer currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YMagnetometer_aio.FirstMagnetometer())

    @classmethod
    def FirstMagnetometerInContext(cls, yctx: YAPIContext) -> Union[YMagnetometer, None]:
        """
        Starts the enumeration of magnetometers currently accessible.
        Use the method YMagnetometer.nextMagnetometer() to iterate on
        next magnetometers.

        @param yctx : a YAPI context.

        @return a pointer to a YMagnetometer object, corresponding to
                the first magnetometer currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YMagnetometer_aio.FirstMagnetometerInContext(yctx))

    def nextMagnetometer(self):
        """
        Continues the enumeration of magnetometers started using yFirstMagnetometer().
        Caution: You can't make any assumption about the returned magnetometers order.
        If you want to find a specific a magnetometer, use Magnetometer.findMagnetometer()
        and a hardwareID or a logical name.

        @return a pointer to a YMagnetometer object, corresponding to
                a magnetometer currently online, or a None pointer
                if there are no more magnetometers to enumerate.
        """
        return self._proxy(type(self), self._aio.nextMagnetometer())

    if not _DYNAMIC_HELPERS:
        def get_bandwidth(self) -> int:
            """
            Returns the measure update frequency, measured in Hz.

            @return an integer corresponding to the measure update frequency, measured in Hz

            On failure, throws an exception or returns YMagnetometer.BANDWIDTH_INVALID.
            """
            return self._run(self._aio.get_bandwidth())

    if not _DYNAMIC_HELPERS:
        def set_bandwidth(self, newval: int) -> int:
            """
            Changes the measure update frequency, measured in Hz. When the
            frequency is lower, the device performs averaging.
            Remember to call the saveToFlash()
            method of the module if the modification must be kept.

            @param newval : an integer corresponding to the measure update frequency, measured in Hz

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_bandwidth(newval))

    if not _DYNAMIC_HELPERS:
        def get_xValue(self) -> float:
            """
            Returns the X component of the magnetic field, as a floating point number.

            @return a floating point number corresponding to the X component of the magnetic field, as a
            floating point number

            On failure, throws an exception or returns YMagnetometer.XVALUE_INVALID.
            """
            return self._run(self._aio.get_xValue())

    if not _DYNAMIC_HELPERS:
        def get_yValue(self) -> float:
            """
            Returns the Y component of the magnetic field, as a floating point number.

            @return a floating point number corresponding to the Y component of the magnetic field, as a
            floating point number

            On failure, throws an exception or returns YMagnetometer.YVALUE_INVALID.
            """
            return self._run(self._aio.get_yValue())

    if not _DYNAMIC_HELPERS:
        def get_zValue(self) -> float:
            """
            Returns the Z component of the magnetic field, as a floating point number.

            @return a floating point number corresponding to the Z component of the magnetic field, as a
            floating point number

            On failure, throws an exception or returns YMagnetometer.ZVALUE_INVALID.
            """
            return self._run(self._aio.get_zValue())

    @classmethod
    def FindMagnetometer(cls, func: str) -> YMagnetometer:
        """
        Retrieves a magnetometer for a given identifier.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the magnetometer is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YMagnetometer.isOnline() to test if the magnetometer is
        indeed online at a given time. In case of ambiguity when looking for
        a magnetometer by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        If a call to this object's is_online() method returns FALSE although
        you are certain that the matching device is plugged, make sure that you did
        call registerHub() at application initialization time.

        @param func : a string that uniquely characterizes the magnetometer, for instance
                Y3DMK002.magnetometer.

        @return a YMagnetometer object allowing you to drive the magnetometer.
        """
        return cls._proxy(cls, YMagnetometer_aio.FindMagnetometer(func))

    @classmethod
    def FindMagnetometerInContext(cls, yctx: YAPIContext, func: str) -> YMagnetometer:
        """
        Retrieves a magnetometer for a given identifier in a YAPI context.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the magnetometer is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YMagnetometer.isOnline() to test if the magnetometer is
        indeed online at a given time. In case of ambiguity when looking for
        a magnetometer by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        @param yctx : a YAPI context
        @param func : a string that uniquely characterizes the magnetometer, for instance
                Y3DMK002.magnetometer.

        @return a YMagnetometer object allowing you to drive the magnetometer.
        """
        return cls._proxy(cls, YMagnetometer_aio.FindMagnetometerInContext(yctx, func))

    if not _IS_MICROPYTHON:
        def registerValueCallback(self, callback: YMagnetometerValueCallback) -> int:
            """
            Registers the callback function that is invoked on every change of advertised value.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and the character string describing
                    the new advertised value.
            @noreturn
            """
            return super().registerValueCallback(callback)

    if not _IS_MICROPYTHON:
        def registerTimedReportCallback(self, callback: YMagnetometerTimedReportCallback) -> int:
            """
            Registers the callback function that is invoked on every periodic timed notification.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and an YMeasure object describing
                    the new advertised value.
            @noreturn
            """
            return super().registerTimedReportCallback(callback)

    # --- (end of YMagnetometer implementation)




# -*- coding: utf-8 -*-
# ********************************************************************
#
#  $Id: svn_id $
#
#  High-level API for YPwmOutput
#
#  - - - - - - - - - License information: - - - - - - - - -
#
#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#
#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#  non-exclusive license to use, modify, copy and integrate this
#  file into your software for the sole purpose of interfacing
#  with Yoctopuce products.
#
#  You may reproduce and distribute copies of this file in
#  source or object form, as long as the sole purpose of this
#  code is to interface with Yoctopuce products. You must retain
#  this notice in the distributed source file.
#
#  You should refer to Yoctopuce General Terms and Conditions
#  for additional information regarding your rights and
#  obligations.
#
#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#  WARRANTY, OR OTHERWISE.
#
# *********************************************************************
"""
Yoctopuce library: High-level API for YPwmOutput
version: PATCH_WITH_VERSION
requires: yocto_pwmoutput_aio
requires: yocto_api
"""
from __future__ import annotations

import sys

# On MicroPython, code below will be wiped out at compile time
if sys.implementation.name != "micropython":
    # In CPython, enable edit-time type checking, including Final declaration
    from typing import Union, Final
    from collections.abc import Callable, Awaitable
    from .yocto_api import _IS_MICROPYTHON, _DYNAMIC_HELPERS
else:
    # In our micropython VM, common generic types are global built-ins
    # Others such as TypeVar should be avoided when using micropython,
    # as they produce overhead in runtime code
    # Final is translated into const() expressions before compilation
    _IS_MICROPYTHON: Final[bool] = True # noqa
    _DYNAMIC_HELPERS: Final[bool] = True # noqa

from .yocto_pwmoutput_aio import YPwmOutput as YPwmOutput_aio
from .yocto_api import (
    YAPIContext, YAPI, YFunction
)

# --- (YPwmOutput class start)
if not _IS_MICROPYTHON:
    # For CPython, use strongly typed callback types
    try:
        YPwmOutputValueCallback = Union[Callable[['YPwmOutput', str], Awaitable[None]], None]
    except TypeError:
        YPwmOutputValueCallback = Union[Callable, Awaitable]

# noinspection PyProtectedMember
class YPwmOutput(YFunction):
    """
    The YPwmOutput class allows you to drive a pulse-width modulated output (PWM).
    You can configure the frequency as well as the duty cycle, and set up progressive
    transitions.

    """
    _aio: YPwmOutput_aio
    # --- (end of YPwmOutput class start)
    if not _IS_MICROPYTHON:
        # --- (YPwmOutput return codes)
        FREQUENCY_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        PERIOD_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        DUTYCYCLE_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        PULSEDURATION_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        PWMTRANSITION_INVALID: Final[str] = YAPI.INVALID_STRING
        DUTYCYCLEATPOWERON_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        ENABLED_FALSE: Final[int] = 0
        ENABLED_TRUE: Final[int] = 1
        ENABLED_INVALID: Final[int] = -1
        INVERTEDOUTPUT_FALSE: Final[int] = 0
        INVERTEDOUTPUT_TRUE: Final[int] = 1
        INVERTEDOUTPUT_INVALID: Final[int] = -1
        ENABLEDATPOWERON_FALSE: Final[int] = 0
        ENABLEDATPOWERON_TRUE: Final[int] = 1
        ENABLEDATPOWERON_INVALID: Final[int] = -1
        # --- (end of YPwmOutput return codes)


    # --- (YPwmOutput implementation)

    @classmethod
    def FirstPwmOutput(cls) -> Union[YPwmOutput, None]:
        """
        Starts the enumeration of PWM generators currently accessible.
        Use the method YPwmOutput.nextPwmOutput() to iterate on
        next PWM generators.

        @return a pointer to a YPwmOutput object, corresponding to
                the first PWM generator currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YPwmOutput_aio.FirstPwmOutput())

    @classmethod
    def FirstPwmOutputInContext(cls, yctx: YAPIContext) -> Union[YPwmOutput, None]:
        """
        Starts the enumeration of PWM generators currently accessible.
        Use the method YPwmOutput.nextPwmOutput() to iterate on
        next PWM generators.

        @param yctx : a YAPI context.

        @return a pointer to a YPwmOutput object, corresponding to
                the first PWM generator currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YPwmOutput_aio.FirstPwmOutputInContext(yctx))

    def nextPwmOutput(self):
        """
        Continues the enumeration of PWM generators started using yFirstPwmOutput().
        Caution: You can't make any assumption about the returned PWM generators order.
        If you want to find a specific a PWM generator, use PwmOutput.findPwmOutput()
        and a hardwareID or a logical name.

        @return a pointer to a YPwmOutput object, corresponding to
                a PWM generator currently online, or a None pointer
                if there are no more PWM generators to enumerate.
        """
        return self._proxy(type(self), self._aio.nextPwmOutput())

    if not _DYNAMIC_HELPERS:
        def get_enabled(self) -> int:
            """
            Returns the state of the PWM generators.

            @return either YPwmOutput.ENABLED_FALSE or YPwmOutput.ENABLED_TRUE, according to the state of the PWM generators

            On failure, throws an exception or returns YPwmOutput.ENABLED_INVALID.
            """
            return self._run(self._aio.get_enabled())

    if not _DYNAMIC_HELPERS:
        def set_enabled(self, newval: int) -> int:
            """
            Stops or starts the PWM.

            @param newval : either YPwmOutput.ENABLED_FALSE or YPwmOutput.ENABLED_TRUE

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_enabled(newval))

    if not _DYNAMIC_HELPERS:
        def set_frequency(self, newval: float) -> int:
            """
            Changes the PWM frequency. The duty cycle is kept unchanged thanks to an
            automatic pulse width change, in other words, the change will not be applied
            before the end of the current period. This can significantly affect reaction
            time at low frequencies. If you call the matching module saveToFlash()
            method, the frequency will be kept after a device power cycle.
            To stop the PWM signal, do not set the frequency to zero, use the set_enabled()
            method instead.

            @param newval : a floating point number corresponding to the PWM frequency

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_frequency(newval))

    if not _DYNAMIC_HELPERS:
        def get_frequency(self) -> float:
            """
            Returns the PWM frequency in Hz.

            @return a floating point number corresponding to the PWM frequency in Hz

            On failure, throws an exception or returns YPwmOutput.FREQUENCY_INVALID.
            """
            return self._run(self._aio.get_frequency())

    if not _DYNAMIC_HELPERS:
        def set_period(self, newval: float) -> int:
            """
            Changes the PWM period in milliseconds. Caution: in order to avoid  random truncation of
            the current pulse, the change will not be applied
            before the end of the current period. This can significantly affect reaction
            time at low frequencies. If you call the matching module saveToFlash()
            method, the frequency will be kept after a device power cycle.

            @param newval : a floating point number corresponding to the PWM period in milliseconds

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_period(newval))

    if not _DYNAMIC_HELPERS:
        def get_period(self) -> float:
            """
            Returns the PWM period in milliseconds.

            @return a floating point number corresponding to the PWM period in milliseconds

            On failure, throws an exception or returns YPwmOutput.PERIOD_INVALID.
            """
            return self._run(self._aio.get_period())

    if not _DYNAMIC_HELPERS:
        def set_dutyCycle(self, newval: float) -> int:
            """
            Changes the PWM duty cycle, in per cents.

            @param newval : a floating point number corresponding to the PWM duty cycle, in per cents

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_dutyCycle(newval))

    if not _DYNAMIC_HELPERS:
        def get_dutyCycle(self) -> float:
            """
            Returns the PWM duty cycle, in per cents.

            @return a floating point number corresponding to the PWM duty cycle, in per cents

            On failure, throws an exception or returns YPwmOutput.DUTYCYCLE_INVALID.
            """
            return self._run(self._aio.get_dutyCycle())

    if not _DYNAMIC_HELPERS:
        def set_pulseDuration(self, newval: float) -> int:
            """
            Changes the PWM pulse length, in milliseconds. A pulse length cannot be longer than period,
            otherwise it is truncated.

            @param newval : a floating point number corresponding to the PWM pulse length, in milliseconds

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_pulseDuration(newval))

    if not _DYNAMIC_HELPERS:
        def get_pulseDuration(self) -> float:
            """
            Returns the PWM pulse length in milliseconds, as a floating point number.

            @return a floating point number corresponding to the PWM pulse length in milliseconds, as a
            floating point number

            On failure, throws an exception or returns YPwmOutput.PULSEDURATION_INVALID.
            """
            return self._run(self._aio.get_pulseDuration())

    if not _DYNAMIC_HELPERS:
        def set_pwmTransition(self, newval: str) -> int:
            return self._run(self._aio.set_pwmTransition(newval))

    if not _DYNAMIC_HELPERS:
        def get_invertedOutput(self) -> int:
            """
            Returns true if the output signal is configured as inverted, and false otherwise.

            @return either YPwmOutput.INVERTEDOUTPUT_FALSE or YPwmOutput.INVERTEDOUTPUT_TRUE, according to true
            if the output signal is configured as inverted, and false otherwise

            On failure, throws an exception or returns YPwmOutput.INVERTEDOUTPUT_INVALID.
            """
            return self._run(self._aio.get_invertedOutput())

    if not _DYNAMIC_HELPERS:
        def set_invertedOutput(self, newval: int) -> int:
            """
            Changes the inversion mode of the output signal.
            Remember to call the matching module saveToFlash() method if you want
            the change to be kept after power cycle.

            @param newval : either YPwmOutput.INVERTEDOUTPUT_FALSE or YPwmOutput.INVERTEDOUTPUT_TRUE, according
            to the inversion mode of the output signal

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_invertedOutput(newval))

    if not _DYNAMIC_HELPERS:
        def get_enabledAtPowerOn(self) -> int:
            """
            Returns the state of the PWM at device power on.

            @return either YPwmOutput.ENABLEDATPOWERON_FALSE or YPwmOutput.ENABLEDATPOWERON_TRUE, according to
            the state of the PWM at device power on

            On failure, throws an exception or returns YPwmOutput.ENABLEDATPOWERON_INVALID.
            """
            return self._run(self._aio.get_enabledAtPowerOn())

    if not _DYNAMIC_HELPERS:
        def set_enabledAtPowerOn(self, newval: int) -> int:
            """
            Changes the state of the PWM at device power on. Remember to call the matching module saveToFlash()
            method, otherwise this call will have no effect.

            @param newval : either YPwmOutput.ENABLEDATPOWERON_FALSE or YPwmOutput.ENABLEDATPOWERON_TRUE,
            according to the state of the PWM at device power on

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_enabledAtPowerOn(newval))

    if not _DYNAMIC_HELPERS:
        def set_dutyCycleAtPowerOn(self, newval: float) -> int:
            """
            Changes the PWM duty cycle at device power on. Remember to call the matching
            module saveToFlash() method, otherwise this call will have no effect.

            @param newval : a floating point number corresponding to the PWM duty cycle at device power on

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_dutyCycleAtPowerOn(newval))

    if not _DYNAMIC_HELPERS:
        def get_dutyCycleAtPowerOn(self) -> float:
            """
            Returns the PWM generators duty cycle at device power on as a floating point number between 0 and 100.

            @return a floating point number corresponding to the PWM generators duty cycle at device power on
            as a floating point number between 0 and 100

            On failure, throws an exception or returns YPwmOutput.DUTYCYCLEATPOWERON_INVALID.
            """
            return self._run(self._aio.get_dutyCycleAtPowerOn())

    @classmethod
    def FindPwmOutput(cls, func: str) -> YPwmOutput:
        """
        Retrieves a PWM generator for a given identifier.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the PWM generator is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YPwmOutput.isOnline() to test if the PWM generator is
        indeed online at a given time. In case of ambiguity when looking for
        a PWM generator by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        If a call to this object's is_online() method returns FALSE although
        you are certain that the matching device is plugged, make sure that you did
        call registerHub() at application initialization time.

        @param func : a string that uniquely characterizes the PWM generator, for instance
                YPWMTX01.pwmOutput1.

        @return a YPwmOutput object allowing you to drive the PWM generator.
        """
        return cls._proxy(cls, YPwmOutput_aio.FindPwmOutput(func))

    @classmethod
    def FindPwmOutputInContext(cls, yctx: YAPIContext, func: str) -> YPwmOutput:
        """
        Retrieves a PWM generator for a given identifier in a YAPI context.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the PWM generator is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YPwmOutput.isOnline() to test if the PWM generator is
        indeed online at a given time. In case of ambiguity when looking for
        a PWM generator by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        @param yctx : a YAPI context
        @param func : a string that uniquely characterizes the PWM generator, for instance
                YPWMTX01.pwmOutput1.

        @return a YPwmOutput object allowing you to drive the PWM generator.
        """
        return cls._proxy(cls, YPwmOutput_aio.FindPwmOutputInContext(yctx, func))

    if not _IS_MICROPYTHON:
        def registerValueCallback(self, callback: YPwmOutputValueCallback) -> int:
            """
            Registers the callback function that is invoked on every change of advertised value.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and the character string describing
                    the new advertised value.
            @noreturn
            """
            return super().registerValueCallback(callback)

    if not _DYNAMIC_HELPERS:
        def pulseDurationMove(self, ms_target: float, ms_duration: int) -> int:
            """
            Performs a smooth transition of the pulse duration toward a given value.
            Any period, frequency, duty cycle or pulse width change will cancel any ongoing transition process.

            @param ms_target   : new pulse duration at the end of the transition
                    (floating-point number, representing the pulse duration in milliseconds)
            @param ms_duration : total duration of the transition, in milliseconds

            @return YAPI.SUCCESS when the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.pulseDurationMove(ms_target, ms_duration))

    if not _DYNAMIC_HELPERS:
        def dutyCycleMove(self, target: float, ms_duration: int) -> int:
            """
            Performs a smooth change of the duty cycle toward a given value.
            Any period, frequency, duty cycle or pulse width change will cancel any ongoing transition process.

            @param target      : new duty cycle at the end of the transition
                    (percentage, floating-point number between 0 and 100)
            @param ms_duration : total duration of the transition, in milliseconds

            @return YAPI.SUCCESS when the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.dutyCycleMove(target, ms_duration))

    if not _DYNAMIC_HELPERS:
        def frequencyMove(self, target: float, ms_duration: int) -> int:
            """
            Performs a smooth frequency change toward a given value.
            Any period, frequency, duty cycle or pulse width change will cancel any ongoing transition process.

            @param target      : new frequency at the end of the transition (floating-point number)
            @param ms_duration : total duration of the transition, in milliseconds

            @return YAPI.SUCCESS when the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.frequencyMove(target, ms_duration))

    if not _DYNAMIC_HELPERS:
        def phaseMove(self, target: float, ms_duration: int) -> int:
            """
            Performs a smooth transition toward a specified value of the phase shift between this channel
            and the other channel. The phase shift is executed by slightly changing the frequency
            temporarily during the specified duration. This function only makes sense when both channels
            are running, either at the same frequency, or at a multiple of the channel frequency.
            Any period, frequency, duty cycle or pulse width change will cancel any ongoing transition process.

            @param target      : phase shift at the end of the transition, in milliseconds (floating-point number)
            @param ms_duration : total duration of the transition, in milliseconds

            @return YAPI.SUCCESS when the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.phaseMove(target, ms_duration))

    if not _DYNAMIC_HELPERS:
        def triggerPulsesByDuration(self, ms_target: float, n_pulses: int) -> int:
            """
            Trigger a given number of pulses of specified duration, at current frequency.
            At the end of the pulse train, revert to the original state of the PWM generator.

            @param ms_target : desired pulse duration
                    (floating-point number, representing the pulse duration in milliseconds)
            @param n_pulses  : desired pulse count

            @return YAPI.SUCCESS when the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.triggerPulsesByDuration(ms_target, n_pulses))

    if not _DYNAMIC_HELPERS:
        def triggerPulsesByDutyCycle(self, target: float, n_pulses: int) -> int:
            """
            Trigger a given number of pulses of specified duration, at current frequency.
            At the end of the pulse train, revert to the original state of the PWM generator.

            @param target   : desired duty cycle for the generated pulses
                    (percentage, floating-point number between 0 and 100)
            @param n_pulses : desired pulse count

            @return YAPI.SUCCESS when the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.triggerPulsesByDutyCycle(target, n_pulses))

    if not _DYNAMIC_HELPERS:
        def triggerPulsesByFrequency(self, target: float, n_pulses: int) -> int:
            """
            Trigger a given number of pulses at the specified frequency, using current duty cycle.
            At the end of the pulse train, revert to the original state of the PWM generator.

            @param target   : desired frequency for the generated pulses (floating-point number)
            @param n_pulses : desired pulse count

            @return YAPI.SUCCESS when the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.triggerPulsesByFrequency(target, n_pulses))

    # --- (end of YPwmOutput implementation)




# -*- coding: utf-8 -*-
# ********************************************************************
#
#  $Id: svn_id $
#
#  Asyncio implementation of YLightSensor
#
#  - - - - - - - - - License information: - - - - - - - - -
#
#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#
#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#  non-exclusive license to use, modify, copy and integrate this
#  file into your software for the sole purpose of interfacing
#  with Yoctopuce products.
#
#  You may reproduce and distribute copies of this file in
#  source or object form, as long as the sole purpose of this
#  code is to interface with Yoctopuce products. You must retain
#  this notice in the distributed source file.
#
#  You should refer to Yoctopuce General Terms and Conditions
#  for additional information regarding your rights and
#  obligations.
#
#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#  WARRANTY, OR OTHERWISE.
#
# *********************************************************************
"""
Yoctopuce library: Asyncio implementation of YLightSensor
version: PATCH_WITH_VERSION
requires: yocto_api_aio
"""
from __future__ import annotations

import sys

# On MicroPython, code below will be wiped out at compile time
if sys.implementation.name != "micropython":
    # In CPython, enable edit-time type checking, including Final declaration
    from typing import Any, Union, Final
    from collections.abc import Callable, Awaitable
    from .yocto_api_aio import const, _IS_MICROPYTHON
else:
    # In our micropython VM, common generic types are global built-ins
    # Others such as TypeVar should be avoided when using micropython,
    # as they produce overhead in runtime code
    # Final is translated into const() expressions before compilation
    _IS_MICROPYTHON: Final[bool] = True # noqa

from .yocto_api_aio import (
    YAPIContext, YAPI, YAPI_Exception, HwId, hwid2str, YFunction, YSensor, YMeasure
)

# --- (YLightSensor class start)
if not _IS_MICROPYTHON:
    # For CPython, use strongly typed callback types
    try:
        YLightSensorValueCallback = Union[Callable[['YLightSensor', str], Awaitable[None]], None]
        YLightSensorTimedReportCallback = Union[Callable[['YLightSensor', YMeasure], Awaitable[None]], None]
    except TypeError:
        YLightSensorValueCallback = Union[Callable, Awaitable]
        YLightSensorTimedReportCallback = Union[Callable, Awaitable]

# noinspection PyProtectedMember
class YLightSensor(YSensor):
    """
    The YLightSensor class allows you to read and configure Yoctopuce light sensors.
    It inherits from YSensor class the core functions to read measures,
    to register callback functions, and to access the autonomous datalogger.
    This class adds the ability to easily perform a one-point linear calibration
    to compensate the effect of a glass or filter placed in front of the sensor.
    For some light sensors with several working modes, this class can select the
    desired working mode.

    """
    # --- (end of YLightSensor class start)
    if not _IS_MICROPYTHON:
        # --- (YLightSensor return codes)
        MEASURETYPE_HUMAN_EYE: Final[int] = 0
        MEASURETYPE_WIDE_SPECTRUM: Final[int] = 1
        MEASURETYPE_INFRARED: Final[int] = 2
        MEASURETYPE_HIGH_RATE: Final[int] = 3
        MEASURETYPE_HIGH_ENERGY: Final[int] = 4
        MEASURETYPE_HIGH_RESOLUTION: Final[int] = 5
        MEASURETYPE_INVALID: Final[int] = -1
        # --- (end of YLightSensor return codes)

    # --- (YLightSensor attributes declaration)
    _measureType: int
    _valueCallback: YLightSensorValueCallback
    _timedReportCallback: YLightSensorTimedReportCallback
    # --- (end of YLightSensor attributes declaration)


    def __init__(self, yctx: YAPIContext, func: str):
        super().__init__(yctx, func)
        self._className = 'LightSensor'
        # --- (YLightSensor constructor)
        self._measureType = YLightSensor.MEASURETYPE_INVALID
        # --- (end of YLightSensor constructor)

    # --- (YLightSensor implementation)

    @staticmethod
    def FirstLightSensor() -> Union[YLightSensor, None]:
        """
        Starts the enumeration of light sensors currently accessible.
        Use the method YLightSensor.nextLightSensor() to iterate on
        next light sensors.

        @return a pointer to a YLightSensor object, corresponding to
                the first light sensor currently online, or a None pointer
                if there are none.
        """
        next_hwid: Union[HwId, None] = YAPI._yHash.getFirstHardwareId('LightSensor')
        if not next_hwid:
            return None
        return YLightSensor.FindLightSensor(hwid2str(next_hwid))

    @staticmethod
    def FirstLightSensorInContext(yctx: YAPIContext) -> Union[YLightSensor, None]:
        """
        Starts the enumeration of light sensors currently accessible.
        Use the method YLightSensor.nextLightSensor() to iterate on
        next light sensors.

        @param yctx : a YAPI context.

        @return a pointer to a YLightSensor object, corresponding to
                the first light sensor currently online, or a None pointer
                if there are none.
        """
        next_hwid: Union[HwId, None] = yctx._yHash.getFirstHardwareId('LightSensor')
        if not next_hwid:
            return None
        return YLightSensor.FindLightSensorInContext(yctx, hwid2str(next_hwid))

    def nextLightSensor(self):
        """
        Continues the enumeration of light sensors started using yFirstLightSensor().
        Caution: You can't make any assumption about the returned light sensors order.
        If you want to find a specific a light sensor, use LightSensor.findLightSensor()
        and a hardwareID or a logical name.

        @return a pointer to a YLightSensor object, corresponding to
                a light sensor currently online, or a None pointer
                if there are no more light sensors to enumerate.
        """
        next_hwid: Union[HwId, None] = None
        try:
            hwid: HwId = self._yapi._yHash.resolveHwID(self._className, self._func)
            next_hwid = self._yapi._yHash.getNextHardwareId(self._className, hwid)
        except YAPI_Exception:
            pass
        if not next_hwid:
            return None
        return YLightSensor.FindLightSensorInContext(self._yapi, hwid2str(next_hwid))

    def _parseAttr(self, json_val: dict) -> None:
        if 'measureType' in json_val:
            self._measureType = json_val["measureType"]
        super()._parseAttr(json_val)

    async def set_currentValue(self, newval: float) -> int:
        rest_val = str(int(round(newval * 65536.0, 1)))
        return await self._setAttr("currentValue", rest_val)

    async def calibrate(self, calibratedVal) -> int:
        """
        Changes the sensor-specific calibration parameter so that the current value
        matches a desired target (linear scaling).

        @param calibratedVal : the desired target value.

        Remember to call the saveToFlash() method of the module if the
        modification must be kept.

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(int(round(calibratedVal * 65536.0, 1)))
        return await self._setAttr("currentValue", rest_val)

    async def get_measureType(self) -> int:
        """
        Returns the type of light measure.

        @return a value among YLightSensor.MEASURETYPE_HUMAN_EYE, YLightSensor.MEASURETYPE_WIDE_SPECTRUM,
        YLightSensor.MEASURETYPE_INFRARED, YLightSensor.MEASURETYPE_HIGH_RATE,
        YLightSensor.MEASURETYPE_HIGH_ENERGY and YLightSensor.MEASURETYPE_HIGH_RESOLUTION corresponding to
        the type of light measure

        On failure, throws an exception or returns YLightSensor.MEASURETYPE_INVALID.
        """
        res: int
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YLightSensor.MEASURETYPE_INVALID
        res = self._measureType
        return res

    async def set_measureType(self, newval: int) -> int:
        """
        Changes the light sensor type used in the device. The measure can either
        approximate the response of the human eye, focus on a specific light
        spectrum, depending on the capabilities of the light-sensitive cell.
        Remember to call the saveToFlash() method of the module if the
        modification must be kept.

        @param newval : a value among YLightSensor.MEASURETYPE_HUMAN_EYE,
        YLightSensor.MEASURETYPE_WIDE_SPECTRUM, YLightSensor.MEASURETYPE_INFRARED,
        YLightSensor.MEASURETYPE_HIGH_RATE, YLightSensor.MEASURETYPE_HIGH_ENERGY and
        YLightSensor.MEASURETYPE_HIGH_RESOLUTION corresponding to the light sensor type used in the device

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(newval)
        return await self._setAttr("measureType", rest_val)

    @staticmethod
    def FindLightSensor(func: str) -> YLightSensor:
        """
        Retrieves a light sensor for a given identifier.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the light sensor is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YLightSensor.isOnline() to test if the light sensor is
        indeed online at a given time. In case of ambiguity when looking for
        a light sensor by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        If a call to this object's is_online() method returns FALSE although
        you are certain that the matching device is plugged, make sure that you did
        call registerHub() at application initialization time.

        @param func : a string that uniquely characterizes the light sensor, for instance
                LIGHTMK4.lightSensor.

        @return a YLightSensor object allowing you to drive the light sensor.
        """
        obj: Union[YLightSensor, None]
        obj = YFunction._FindFromCache("LightSensor", func)
        if obj is None:
            obj = YLightSensor(YAPI, func)
            YFunction._AddToCache("LightSensor", func, obj)
        return obj

    @staticmethod
    def FindLightSensorInContext(yctx: YAPIContext, func: str) -> YLightSensor:
        """
        Retrieves a light sensor for a given identifier in a YAPI context.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the light sensor is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YLightSensor.isOnline() to test if the light sensor is
        indeed online at a given time. In case of ambiguity when looking for
        a light sensor by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        @param yctx : a YAPI context
        @param func : a string that uniquely characterizes the light sensor, for instance
                LIGHTMK4.lightSensor.

        @return a YLightSensor object allowing you to drive the light sensor.
        """
        obj: Union[YLightSensor, None]
        obj = YFunction._FindFromCacheInContext(yctx, "LightSensor", func)
        if obj is None:
            obj = YLightSensor(yctx, func)
            YFunction._AddToCache("LightSensor", func, obj)
        return obj

    if not _IS_MICROPYTHON:
        async def registerValueCallback(self, callback: YLightSensorValueCallback) -> int:
            """
            Registers the callback function that is invoked on every change of advertised value.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and the character string describing
                    the new advertised value.
            @noreturn
            """
            return await super().registerValueCallback(callback)

    if not _IS_MICROPYTHON:
        async def registerTimedReportCallback(self, callback: YLightSensorTimedReportCallback) -> int:
            """
            Registers the callback function that is invoked on every periodic timed notification.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and an YMeasure object describing
                    the new advertised value.
            @noreturn
            """
            return await super().registerTimedReportCallback(callback)

    # --- (end of YLightSensor implementation)




# -*- coding: utf-8 -*-
# ********************************************************************
#
#  $Id: svn_id $
#
#  High-level API for YOsControl
#
#  - - - - - - - - - License information: - - - - - - - - -
#
#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#
#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#  non-exclusive license to use, modify, copy and integrate this
#  file into your software for the sole purpose of interfacing
#  with Yoctopuce products.
#
#  You may reproduce and distribute copies of this file in
#  source or object form, as long as the sole purpose of this
#  code is to interface with Yoctopuce products. You must retain
#  this notice in the distributed source file.
#
#  You should refer to Yoctopuce General Terms and Conditions
#  for additional information regarding your rights and
#  obligations.
#
#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#  WARRANTY, OR OTHERWISE.
#
# *********************************************************************
"""
Yoctopuce library: High-level API for YOsControl
version: PATCH_WITH_VERSION
requires: yocto_oscontrol_aio
requires: yocto_api
"""
from __future__ import annotations

import sys

# On MicroPython, code below will be wiped out at compile time
if sys.implementation.name != "micropython":
    # In CPython, enable edit-time type checking, including Final declaration
    from typing import Union, Final
    from collections.abc import Callable, Awaitable
    from .yocto_api import _IS_MICROPYTHON, _DYNAMIC_HELPERS
else:
    # In our micropython VM, common generic types are global built-ins
    # Others such as TypeVar should be avoided when using micropython,
    # as they produce overhead in runtime code
    # Final is translated into const() expressions before compilation
    _IS_MICROPYTHON: Final[bool] = True # noqa
    _DYNAMIC_HELPERS: Final[bool] = True # noqa

from .yocto_oscontrol_aio import YOsControl as YOsControl_aio
from .yocto_api import (
    YAPIContext, YAPI, YFunction
)

# --- (YOsControl class start)
if not _IS_MICROPYTHON:
    # For CPython, use strongly typed callback types
    try:
        YOsControlValueCallback = Union[Callable[['YOsControl', str], Awaitable[None]], None]
    except TypeError:
        YOsControlValueCallback = Union[Callable, Awaitable]

# noinspection PyProtectedMember
class YOsControl(YFunction):
    """
    The YOScontrol class provides some control over the operating system running a VirtualHub.
    YOsControl is available on VirtualHub software only. This feature must be activated at the VirtualHub
    start up with -o option.

    """
    _aio: YOsControl_aio
    # --- (end of YOsControl class start)
    if not _IS_MICROPYTHON:
        # --- (YOsControl return codes)
        SHUTDOWNCOUNTDOWN_INVALID: Final[int] = YAPI.INVALID_INT
        # --- (end of YOsControl return codes)


    # --- (YOsControl implementation)

    @classmethod
    def FirstOsControl(cls) -> Union[YOsControl, None]:
        """
        Starts the enumeration of OS control currently accessible.
        Use the method YOsControl.nextOsControl() to iterate on
        next OS control.

        @return a pointer to a YOsControl object, corresponding to
                the first OS control currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YOsControl_aio.FirstOsControl())

    @classmethod
    def FirstOsControlInContext(cls, yctx: YAPIContext) -> Union[YOsControl, None]:
        """
        Starts the enumeration of OS control currently accessible.
        Use the method YOsControl.nextOsControl() to iterate on
        next OS control.

        @param yctx : a YAPI context.

        @return a pointer to a YOsControl object, corresponding to
                the first OS control currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YOsControl_aio.FirstOsControlInContext(yctx))

    def nextOsControl(self):
        """
        Continues the enumeration of OS control started using yFirstOsControl().
        Caution: You can't make any assumption about the returned OS control order.
        If you want to find a specific OS control, use OsControl.findOsControl()
        and a hardwareID or a logical name.

        @return a pointer to a YOsControl object, corresponding to
                OS control currently online, or a None pointer
                if there are no more OS control to enumerate.
        """
        return self._proxy(type(self), self._aio.nextOsControl())

    if not _DYNAMIC_HELPERS:
        def get_shutdownCountdown(self) -> int:
            """
            Returns the remaining number of seconds before the OS shutdown, or zero when no
            shutdown has been scheduled.

            @return an integer corresponding to the remaining number of seconds before the OS shutdown, or zero when no
                    shutdown has been scheduled

            On failure, throws an exception or returns YOsControl.SHUTDOWNCOUNTDOWN_INVALID.
            """
            return self._run(self._aio.get_shutdownCountdown())

    if not _DYNAMIC_HELPERS:
        def set_shutdownCountdown(self, newval: int) -> int:
            return self._run(self._aio.set_shutdownCountdown(newval))

    @classmethod
    def FindOsControl(cls, func: str) -> YOsControl:
        """
        Retrieves OS control for a given identifier.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the OS control is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YOsControl.isOnline() to test if the OS control is
        indeed online at a given time. In case of ambiguity when looking for
        OS control by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        If a call to this object's is_online() method returns FALSE although
        you are certain that the matching device is plugged, make sure that you did
        call registerHub() at application initialization time.

        @param func : a string that uniquely characterizes the OS control, for instance
                MyDevice.osControl.

        @return a YOsControl object allowing you to drive the OS control.
        """
        return cls._proxy(cls, YOsControl_aio.FindOsControl(func))

    @classmethod
    def FindOsControlInContext(cls, yctx: YAPIContext, func: str) -> YOsControl:
        """
        Retrieves OS control for a given identifier in a YAPI context.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the OS control is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YOsControl.isOnline() to test if the OS control is
        indeed online at a given time. In case of ambiguity when looking for
        OS control by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        @param yctx : a YAPI context
        @param func : a string that uniquely characterizes the OS control, for instance
                MyDevice.osControl.

        @return a YOsControl object allowing you to drive the OS control.
        """
        return cls._proxy(cls, YOsControl_aio.FindOsControlInContext(yctx, func))

    if not _IS_MICROPYTHON:
        def registerValueCallback(self, callback: YOsControlValueCallback) -> int:
            """
            Registers the callback function that is invoked on every change of advertised value.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and the character string describing
                    the new advertised value.
            @noreturn
            """
            return super().registerValueCallback(callback)

    if not _DYNAMIC_HELPERS:
        def shutdown(self, secBeforeShutDown: int) -> int:
            """
            Schedules an OS shutdown after a given number of seconds.

            @param secBeforeShutDown : number of seconds before shutdown

            @return YAPI.SUCCESS when the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.shutdown(secBeforeShutDown))

    if not _DYNAMIC_HELPERS:
        def reboot(self, secBeforeReboot: int) -> int:
            """
            Schedules an OS reboot after a given number of seconds.

            @param secBeforeReboot : number of seconds before reboot

            @return YAPI.SUCCESS when the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.reboot(secBeforeReboot))

    # --- (end of YOsControl implementation)




# -*- coding: utf-8 -*-
# ********************************************************************
#
#  $Id: svn_id $
#
#  High-level API for YPowerOutput
#
#  - - - - - - - - - License information: - - - - - - - - -
#
#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#
#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#  non-exclusive license to use, modify, copy and integrate this
#  file into your software for the sole purpose of interfacing
#  with Yoctopuce products.
#
#  You may reproduce and distribute copies of this file in
#  source or object form, as long as the sole purpose of this
#  code is to interface with Yoctopuce products. You must retain
#  this notice in the distributed source file.
#
#  You should refer to Yoctopuce General Terms and Conditions
#  for additional information regarding your rights and
#  obligations.
#
#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#  WARRANTY, OR OTHERWISE.
#
# *********************************************************************
"""
Yoctopuce library: High-level API for YPowerOutput
version: PATCH_WITH_VERSION
requires: yocto_poweroutput_aio
requires: yocto_api
"""
from __future__ import annotations

import sys

# On MicroPython, code below will be wiped out at compile time
if sys.implementation.name != "micropython":
    # In CPython, enable edit-time type checking, including Final declaration
    from typing import Union, Final
    from collections.abc import Callable, Awaitable
    from .yocto_api import _IS_MICROPYTHON, _DYNAMIC_HELPERS
else:
    # In our micropython VM, common generic types are global built-ins
    # Others such as TypeVar should be avoided when using micropython,
    # as they produce overhead in runtime code
    # Final is translated into const() expressions before compilation
    _IS_MICROPYTHON: Final[bool] = True # noqa
    _DYNAMIC_HELPERS: Final[bool] = True # noqa

from .yocto_poweroutput_aio import YPowerOutput as YPowerOutput_aio
from .yocto_api import (
    YAPIContext, YAPI, YFunction
)

# --- (YPowerOutput class start)
if not _IS_MICROPYTHON:
    # For CPython, use strongly typed callback types
    try:
        YPowerOutputValueCallback = Union[Callable[['YPowerOutput', str], Awaitable[None]], None]
    except TypeError:
        YPowerOutputValueCallback = Union[Callable, Awaitable]

# noinspection PyProtectedMember
class YPowerOutput(YFunction):
    """
    The YPowerOutput class allows you to control
    the power output featured on some Yoctopuce devices.

    """
    _aio: YPowerOutput_aio
    # --- (end of YPowerOutput class start)
    if not _IS_MICROPYTHON:
        # --- (YPowerOutput return codes)
        VOLTAGE_OFF: Final[int] = 0
        VOLTAGE_OUT3V3: Final[int] = 1
        VOLTAGE_OUT5V: Final[int] = 2
        VOLTAGE_OUT4V7: Final[int] = 3
        VOLTAGE_OUT1V8: Final[int] = 4
        VOLTAGE_INVALID: Final[int] = -1
        # --- (end of YPowerOutput return codes)


    # --- (YPowerOutput implementation)

    @classmethod
    def FirstPowerOutput(cls) -> Union[YPowerOutput, None]:
        """
        Starts the enumeration of power output currently accessible.
        Use the method YPowerOutput.nextPowerOutput() to iterate on
        next power output.

        @return a pointer to a YPowerOutput object, corresponding to
                the first power output currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YPowerOutput_aio.FirstPowerOutput())

    @classmethod
    def FirstPowerOutputInContext(cls, yctx: YAPIContext) -> Union[YPowerOutput, None]:
        """
        Starts the enumeration of power output currently accessible.
        Use the method YPowerOutput.nextPowerOutput() to iterate on
        next power output.

        @param yctx : a YAPI context.

        @return a pointer to a YPowerOutput object, corresponding to
                the first power output currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YPowerOutput_aio.FirstPowerOutputInContext(yctx))

    def nextPowerOutput(self):
        """
        Continues the enumeration of power output started using yFirstPowerOutput().
        Caution: You can't make any assumption about the returned power output order.
        If you want to find a specific a power output, use PowerOutput.findPowerOutput()
        and a hardwareID or a logical name.

        @return a pointer to a YPowerOutput object, corresponding to
                a power output currently online, or a None pointer
                if there are no more power output to enumerate.
        """
        return self._proxy(type(self), self._aio.nextPowerOutput())

    if not _DYNAMIC_HELPERS:
        def get_voltage(self) -> int:
            """
            Returns the voltage on the power output featured by the module.

            @return a value among YPowerOutput.VOLTAGE_OFF, YPowerOutput.VOLTAGE_OUT3V3,
            YPowerOutput.VOLTAGE_OUT5V, YPowerOutput.VOLTAGE_OUT4V7 and YPowerOutput.VOLTAGE_OUT1V8
            corresponding to the voltage on the power output featured by the module

            On failure, throws an exception or returns YPowerOutput.VOLTAGE_INVALID.
            """
            return self._run(self._aio.get_voltage())

    if not _DYNAMIC_HELPERS:
        def set_voltage(self, newval: int) -> int:
            """
            Changes the voltage on the power output provided by the
            module. Remember to call the saveToFlash() method of the module if the
            modification must be kept.

            @param newval : a value among YPowerOutput.VOLTAGE_OFF, YPowerOutput.VOLTAGE_OUT3V3,
            YPowerOutput.VOLTAGE_OUT5V, YPowerOutput.VOLTAGE_OUT4V7 and YPowerOutput.VOLTAGE_OUT1V8
            corresponding to the voltage on the power output provided by the
                    module

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_voltage(newval))

    @classmethod
    def FindPowerOutput(cls, func: str) -> YPowerOutput:
        """
        Retrieves a power output for a given identifier.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the power output is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YPowerOutput.isOnline() to test if the power output is
        indeed online at a given time. In case of ambiguity when looking for
        a power output by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        If a call to this object's is_online() method returns FALSE although
        you are certain that the matching device is plugged, make sure that you did
        call registerHub() at application initialization time.

        @param func : a string that uniquely characterizes the power output, for instance
                YI2CMK01.powerOutput.

        @return a YPowerOutput object allowing you to drive the power output.
        """
        return cls._proxy(cls, YPowerOutput_aio.FindPowerOutput(func))

    @classmethod
    def FindPowerOutputInContext(cls, yctx: YAPIContext, func: str) -> YPowerOutput:
        """
        Retrieves a power output for a given identifier in a YAPI context.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the power output is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YPowerOutput.isOnline() to test if the power output is
        indeed online at a given time. In case of ambiguity when looking for
        a power output by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        @param yctx : a YAPI context
        @param func : a string that uniquely characterizes the power output, for instance
                YI2CMK01.powerOutput.

        @return a YPowerOutput object allowing you to drive the power output.
        """
        return cls._proxy(cls, YPowerOutput_aio.FindPowerOutputInContext(yctx, func))

    if not _IS_MICROPYTHON:
        def registerValueCallback(self, callback: YPowerOutputValueCallback) -> int:
            """
            Registers the callback function that is invoked on every change of advertised value.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and the character string describing
                    the new advertised value.
            @noreturn
            """
            return super().registerValueCallback(callback)

    # --- (end of YPowerOutput implementation)




#!/usr/bin/python
# -*- coding: utf-8 -*-
import os, sys

# add ../../Sources to the PYTHONPATH
sys.path.append(os.path.join("..", "..", "Sources"))

from yocto_api import *


def upgradeSerialList(all_serials):
    for serial in all_serials:
        module = YModule.FindModule(serial)
        product = module.get_productName()
        serial = module.get_serialNumber()
        current = module.get_firmwareRelease()
        # check if a new firmware is available on yoctopuce.com
        new_firmware = module.checkFirmware("www.yoctopuce.com", True)
        if new_firmware == "":
            print(product + " " + serial + "(rev=" + current + ") is up to date")
        else:
            print(product + " " + serial + "(rev=" + current + ") need be updated with firmware : ")
            print("    " + new_firmware)
            # execute the firmware upgrade
            update = module.updateFirmware(new_firmware)
            status = update.startUpdate()
            while 100 > status >= 0:
                new_status = update.get_progress()
                if new_status != status:
                    print(str(new_status) + "% " + update.get_progressMessage())
                YAPI.Sleep(500, errmsg)
                status = new_status
            if status < 0:
                print("Firmware Update failed: " + update.get_progressMessage())
                exit(1)
            else:
                if module.isOnline():
                    print(str(status) + "% Firmware Updated Successfully!")
                else:
                    print(str(status) + " Firmware Update failed: module " + serial + "is not online")
                    exit(1)


errmsg = YRefParam()
# Setup the API to use local USB devices
if YAPI.RegisterHub("usb", errmsg) != YAPI.SUCCESS:
    sys.exit("init error" + errmsg.value)

i = 1
for i in range(1, len(sys.argv)):
    print("Update module connected to hub " + sys.argv[i])
    # Setup the API to use local USB devices
    if YAPI.RegisterHub(sys.argv[i], errmsg) != YAPI.SUCCESS:
        sys.exit("init error" + errmsg.value)

hubs = []
shield = []
devices = []
# fist step construct the list of all hub /shield and devices connected
module = YModule.FirstModule()
while module is not None:
    product = module.get_productName()
    serial = module.get_serialNumber()
    if product == "YoctoHub-Shield":
        shield.append(serial)
    elif product[0:8] == "YoctoHub":
        hubs.append(serial)
    elif product != "VirtualHub":
        devices.append(serial)
    module = module.nextModule()
# fist upgrades all Hubs...
upgradeSerialList(hubs)
# ... then all shield..
upgradeSerialList(shield)
# ... and finally all devices
upgradeSerialList(devices)
print("All devices are now up to date")
YAPI.FreeAPI()



# ********************************************************************
#
#  $Id: helloworld.py 32630 2018-10-10 14:11:07Z seb $
#
#  An example that show how to use a  Yocto-Altimeter
#
#  You can find more information on our web site:
#   Yocto-Altimeter documentation:
#      https://www.yoctopuce.com/EN/products/yocto-altimeter/doc.html
#   Python API Reference:
#      https://www.yoctopuce.com/EN/doc/reference/yoctolib-python-EN.html
#
# *********************************************************************

#!/usr/bin/python
# -*- coding: utf-8 -*-
import os, sys
# add ../../Sources to the PYTHONPATH
sys.path.append(os.path.join("..", "..", "Sources"))

from yocto_api import *
from yocto_altitude import *
from yocto_temperature import *
from yocto_pressure import *


def usage():
    scriptname = os.path.basename(sys.argv[0])
    print("Usage:")
    print(scriptname + ' <serial_number>')
    print(scriptname + ' <logical_name>')
    print(scriptname + ' any  ')
    sys.exit()


def die(msg):
    sys.exit(msg + ' (check USB cable)')


errmsg = YRefParam()

if len(sys.argv) < 2:
    usage()

target = sys.argv[1]

# Setup the API to use local USB devices
if YAPI.RegisterHub("usb", errmsg) != YAPI.SUCCESS:
    sys.exit("init error" + errmsg.value)

if target == 'any':
    # retreive any altitude sensor
    sensor = YAltitude.FirstAltitude()
    if sensor is None:
        die('No module connected')
    m = sensor.get_module()
    target = m.get_serialNumber()

else:
    m = YModule.FindModule(target)

if not m.isOnline():
    die('device not connected')

altSensor = YAltitude.FindAltitude(target + '.altitude')
pressSensor = YPressure.FindPressure(target + '.pressure')
tempSensor = YTemperature.FindTemperature(target + '.temperature')

while altSensor.isOnline():
    print("%4.1f" % altSensor.get_currentValue() + "m (QNH=" \
          + "%4.1f" % altSensor.get_qnh() + "hPa) " \
          + "%4.1f" % pressSensor.get_currentValue() + "hPa  " \
          + "%2.0f" % tempSensor.get_currentValue() + "deg C " \
          + "(Ctrl-c to stop)  ")
    YAPI.Sleep(1000)
YAPI.FreeAPI()


# -*- coding: utf-8 -*-
# ********************************************************************
#
#  $Id: yocto_wireless.py 66072 2025-04-30 06:59:12Z mvuilleu $
#
#  Implements the asyncio YWireless API for Wireless functions
#
#  - - - - - - - - - License information: - - - - - - - - -
#
#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#
#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#  non-exclusive license to use, modify, copy and integrate this
#  file into your software for the sole purpose of interfacing
#  with Yoctopuce products.
#
#  You may reproduce and distribute copies of this file in
#  source or object form, as long as the sole purpose of this
#  code is to interface with Yoctopuce products. You must retain
#  this notice in the distributed source file.
#
#  You should refer to Yoctopuce General Terms and Conditions
#  for additional information regarding your rights and
#  obligations.
#
#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#  WARRANTY, OR OTHERWISE.
#
# *********************************************************************
"""
Yoctopuce library: High-level API for YWireless
version: PATCH_WITH_VERSION
requires: yocto_api
requires: yocto_wireless_aio
"""
from __future__ import annotations
import sys

# On MicroPython, code below will be wiped out at compile time
if sys.implementation.name != "micropython":
    # In CPython, enable edit-time type checking, including Final declaration
    from typing import Any, Union, Final
    from collections.abc import Callable, Awaitable
    from .yocto_api import const, _IS_MICROPYTHON, _DYNAMIC_HELPERS
else:
    # In our micropython VM, common generic types are global built-ins
    # Others such as TypeVar should be avoided when using micropython,
    # as they produce overhead in runtime code
    # Final is translated into const() expressions before compilation
    _IS_MICROPYTHON: Final[bool] = True
    _DYNAMIC_HELPERS: Final[bool] = True

from .yocto_wireless_aio import (
    YWireless as YWireless_aio,
    YWlanRecord
)
from .yocto_api import (
    YAPIContext, YAPI, YFunction
)

# --- (generated code: YWireless class start)
if not _IS_MICROPYTHON:
    # For CPython, use strongly typed callback types
    try:
        YWirelessValueCallback = Union[Callable[['YWireless', str], Awaitable[None]], None]
    except TypeError:
        YWirelessValueCallback = Union[Callable, Awaitable]

# noinspection PyProtectedMember
class YWireless(YFunction):
    """
    The YWireless class provides control over wireless network parameters
    and status for devices that are wireless-enabled.
    Note that TCP/IP parameters are configured separately, using class YNetwork.

    """
    _aio: YWireless_aio
    # --- (end of generated code: YWireless class start)
    if not _IS_MICROPYTHON:
        # --- (generated code: YWireless return codes)
        LINKQUALITY_INVALID: Final[int] = YAPI.INVALID_UINT
        SSID_INVALID: Final[str] = YAPI.INVALID_STRING
        CHANNEL_INVALID: Final[int] = YAPI.INVALID_UINT
        MESSAGE_INVALID: Final[str] = YAPI.INVALID_STRING
        WLANCONFIG_INVALID: Final[str] = YAPI.INVALID_STRING
        SECURITY_UNKNOWN: Final[int] = 0
        SECURITY_OPEN: Final[int] = 1
        SECURITY_WEP: Final[int] = 2
        SECURITY_WPA: Final[int] = 3
        SECURITY_WPA2: Final[int] = 4
        SECURITY_INVALID: Final[int] = -1
        WLANSTATE_DOWN: Final[int] = 0
        WLANSTATE_SCANNING: Final[int] = 1
        WLANSTATE_CONNECTED: Final[int] = 2
        WLANSTATE_REJECTED: Final[int] = 3
        WLANSTATE_INVALID: Final[int] = -1
        # --- (end of generated code: YWireless return codes)

    # --- (generated code: YWireless attributes declaration)
    _linkQuality: int
    _ssid: str
    _channel: int
    _security: int
    _message: str
    _wlanConfig: str
    _wlanState: int
    _valueCallbackWireless: YWirelessValueCallback
    # --- (end of generated code: YWireless attributes declaration)

    # --- (generated code: YWireless implementation)

    @classmethod
    def FirstWireless(cls) -> Union[YWireless, None]:
        """
        Starts the enumeration of wireless LAN interfaces currently accessible.
        Use the method YWireless.nextWireless() to iterate on
        next wireless LAN interfaces.

        @return a pointer to a YWireless object, corresponding to
                the first wireless LAN interface currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YWireless_aio.FirstWireless())

    @classmethod
    def FirstWirelessInContext(cls, yctx: YAPIContext) -> Union[YWireless, None]:
        """
        Starts the enumeration of wireless LAN interfaces currently accessible.
        Use the method YWireless.nextWireless() to iterate on
        next wireless LAN interfaces.

        @param yctx : a YAPI context.

        @return a pointer to a YWireless object, corresponding to
                the first wireless LAN interface currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YWireless_aio.FirstWirelessInContext(yctx))

    def nextWireless(self):
        """
        Continues the enumeration of wireless LAN interfaces started using yFirstWireless().
        Caution: You can't make any assumption about the returned wireless LAN interfaces order.
        If you want to find a specific a wireless LAN interface, use Wireless.findWireless()
        and a hardwareID or a logical name.

        @return a pointer to a YWireless object, corresponding to
                a wireless LAN interface currently online, or a None pointer
                if there are no more wireless LAN interfaces to enumerate.
        """
        return self._proxy(type(self), self._aio.nextWireless())

    if not _DYNAMIC_HELPERS:
        def get_linkQuality(self) -> int:
            """
            Returns the link quality, expressed in percent.

            @return an integer corresponding to the link quality, expressed in percent

            On failure, throws an exception or returns YWireless.LINKQUALITY_INVALID.
            """
            return self._run(self._aio.get_linkQuality())

    if not _DYNAMIC_HELPERS:
        def get_ssid(self) -> str:
            """
            Returns the wireless network name (SSID).

            @return a string corresponding to the wireless network name (SSID)

            On failure, throws an exception or returns YWireless.SSID_INVALID.
            """
            return self._run(self._aio.get_ssid())

    if not _DYNAMIC_HELPERS:
        def get_channel(self) -> int:
            """
            Returns the 802.11 channel currently used, or 0 when the selected network has not been found.

            @return an integer corresponding to the 802.11 channel currently used, or 0 when the selected
            network has not been found

            On failure, throws an exception or returns YWireless.CHANNEL_INVALID.
            """
            return self._run(self._aio.get_channel())

    if not _DYNAMIC_HELPERS:
        def get_security(self) -> int:
            """
            Returns the security algorithm used by the selected wireless network.

            @return a value among YWireless.SECURITY_UNKNOWN, YWireless.SECURITY_OPEN, YWireless.SECURITY_WEP,
            YWireless.SECURITY_WPA and YWireless.SECURITY_WPA2 corresponding to the security algorithm used by
            the selected wireless network

            On failure, throws an exception or returns YWireless.SECURITY_INVALID.
            """
            return self._run(self._aio.get_security())

    if not _DYNAMIC_HELPERS:
        def get_message(self) -> str:
            """
            Returns the latest status message from the wireless interface.

            @return a string corresponding to the latest status message from the wireless interface

            On failure, throws an exception or returns YWireless.MESSAGE_INVALID.
            """
            return self._run(self._aio.get_message())

    if not _DYNAMIC_HELPERS:
        def set_wlanConfig(self, newval: str) -> int:
            return self._run(self._aio.set_wlanConfig(newval))

    if not _DYNAMIC_HELPERS:
        def get_wlanState(self) -> int:
            """
            Returns the current state of the wireless interface. The state YWireless.WLANSTATE_DOWN means that
            the network interface is
            not connected to a network. The state YWireless.WLANSTATE_SCANNING means that the network interface
            is scanning available
            frequencies. During this stage, the device is not reachable, and the network settings are not yet
            applied. The state
            YWireless.WLANSTATE_CONNECTED means that the network settings have been successfully applied ant
            that the device is reachable
            from the wireless network. If the device is configured to use ad-hoc or Soft AP mode, it means that
            the wireless network
            is up and that other devices can join the network. The state YWireless.WLANSTATE_REJECTED means
            that the network interface has
            not been able to join the requested network. The description of the error can be obtain with the
            get_message() method.

            @return a value among YWireless.WLANSTATE_DOWN, YWireless.WLANSTATE_SCANNING,
            YWireless.WLANSTATE_CONNECTED and YWireless.WLANSTATE_REJECTED corresponding to the current state
            of the wireless interface

            On failure, throws an exception or returns YWireless.WLANSTATE_INVALID.
            """
            return self._run(self._aio.get_wlanState())

    @classmethod
    def FindWireless(cls, func: str) -> YWireless:
        """
        Retrieves a wireless LAN interface for a given identifier.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the wireless LAN interface is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YWireless.isOnline() to test if the wireless LAN interface is
        indeed online at a given time. In case of ambiguity when looking for
        a wireless LAN interface by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        If a call to this object's is_online() method returns FALSE although
        you are certain that the matching device is plugged, make sure that you did
        call registerHub() at application initialization time.

        @param func : a string that uniquely characterizes the wireless LAN interface, for instance
                YHUBWLN1.wireless.

        @return a YWireless object allowing you to drive the wireless LAN interface.
        """
        return cls._proxy(cls, YWireless_aio.FindWireless(func))

    @classmethod
    def FindWirelessInContext(cls, yctx: YAPIContext, func: str) -> YWireless:
        """
        Retrieves a wireless LAN interface for a given identifier in a YAPI context.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the wireless LAN interface is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YWireless.isOnline() to test if the wireless LAN interface is
        indeed online at a given time. In case of ambiguity when looking for
        a wireless LAN interface by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        @param yctx : a YAPI context
        @param func : a string that uniquely characterizes the wireless LAN interface, for instance
                YHUBWLN1.wireless.

        @return a YWireless object allowing you to drive the wireless LAN interface.
        """
        return cls._proxy(cls, YWireless_aio.FindWirelessInContext(yctx, func))

    if not _IS_MICROPYTHON:
        def registerValueCallback(self, callback: YWirelessValueCallback) -> int:
            """
            Registers the callback function that is invoked on every change of advertised value.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and the character string describing
                    the new advertised value.
            @noreturn
            """
            return super().registerValueCallback(callback)

    if not _DYNAMIC_HELPERS:
        def startWlanScan(self) -> int:
            """
            Triggers a scan of the wireless frequency and builds the list of available networks.
            The scan forces a disconnection from the current network. At then end of the process, the
            the network interface attempts to reconnect to the previous network. During the scan, the wlanState
            switches to YWireless.WLANSTATE_DOWN, then to YWireless.WLANSTATE_SCANNING. When the scan is completed,
            get_wlanState() returns either YWireless.WLANSTATE_DOWN or YWireless.WLANSTATE_SCANNING. At this
            point, the list of detected network can be retrieved with the get_detectedWlans() method.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.startWlanScan())

    if not _DYNAMIC_HELPERS:
        def joinNetwork(self, ssid: str, securityKey: str) -> int:
            """
            Changes the configuration of the wireless lan interface to connect to an existing
            access point (infrastructure mode).
            Remember to call the saveToFlash() method and then to reboot the module to apply this setting.

            @param ssid : the name of the network to connect to
            @param securityKey : the network key, as a character string

            @return YAPI.SUCCESS when the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.joinNetwork(ssid, securityKey))

    if not _DYNAMIC_HELPERS:
        def adhocNetwork(self, ssid: str, securityKey: str) -> int:
            """
            Changes the configuration of the wireless lan interface to create an ad-hoc
            wireless network, without using an access point. On the YoctoHub-Wireless-g
            and YoctoHub-Wireless-n,
            you should use softAPNetwork() instead, which emulates an access point
            (Soft AP) which is more efficient and more widely supported than ad-hoc networks.

            When a security key is specified for an ad-hoc network, the network is protected
            by a WEP40 key (5 characters or 10 hexadecimal digits) or WEP128 key (13 characters
            or 26 hexadecimal digits). It is recommended to use a well-randomized WEP128 key
            using 26 hexadecimal digits to maximize security.
            Remember to call the saveToFlash() method and then to reboot the module
            to apply this setting.

            @param ssid : the name of the network to connect to
            @param securityKey : the network key, as a character string

            @return YAPI.SUCCESS when the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.adhocNetwork(ssid, securityKey))

    if not _DYNAMIC_HELPERS:
        def softAPNetwork(self, ssid: str, securityKey: str) -> int:
            """
            Changes the configuration of the wireless lan interface to create a new wireless
            network by emulating a WiFi access point (Soft AP). This function can only be
            used with the YoctoHub-Wireless-g and the YoctoHub-Wireless-n.

            On the YoctoHub-Wireless-g, when a security key is specified for a SoftAP network,
            the network is protected by a WEP40 key (5 characters or 10 hexadecimal digits) or
            WEP128 key (13 characters or 26 hexadecimal digits). It is recommended to use a
            well-randomized WEP128 key using 26 hexadecimal digits to maximize security.

            On the YoctoHub-Wireless-n, when a security key is specified for a SoftAP network,
            the network will be protected by WPA2.

            Remember to call the saveToFlash() method and then to reboot the module to apply this setting.

            @param ssid : the name of the network to connect to
            @param securityKey : the network key, as a character string

            @return YAPI.SUCCESS when the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.softAPNetwork(ssid, securityKey))

    if not _DYNAMIC_HELPERS:
        def get_detectedWlans(self) -> list[YWlanRecord]:
            """
            Returns a list of YWlanRecord objects that describe detected Wireless networks.
            This list is not updated when the module is already connected to an access point (infrastructure mode).
            To force an update of this list, startWlanScan() must be called.
            Note that an languages without garbage collections, the returned list must be freed by the caller.

            @return a list of YWlanRecord objects, containing the SSID, channel,
                    link quality and the type of security of the wireless network.

            On failure, throws an exception or returns an empty list.
            """
            return self._run(self._aio.get_detectedWlans())

    # --- (end of generated code: YWireless implementation)




# -*- coding: utf-8 -*-
# ********************************************************************
#
#  $Id: svn_id $
#
#  High-level API for YGenericSensor
#
#  - - - - - - - - - License information: - - - - - - - - -
#
#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#
#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#  non-exclusive license to use, modify, copy and integrate this
#  file into your software for the sole purpose of interfacing
#  with Yoctopuce products.
#
#  You may reproduce and distribute copies of this file in
#  source or object form, as long as the sole purpose of this
#  code is to interface with Yoctopuce products. You must retain
#  this notice in the distributed source file.
#
#  You should refer to Yoctopuce General Terms and Conditions
#  for additional information regarding your rights and
#  obligations.
#
#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#  WARRANTY, OR OTHERWISE.
#
# *********************************************************************
"""
Yoctopuce library: High-level API for YGenericSensor
version: PATCH_WITH_VERSION
requires: yocto_genericsensor_aio
requires: yocto_api
"""
from __future__ import annotations

import sys

# On MicroPython, code below will be wiped out at compile time
if sys.implementation.name != "micropython":
    # In CPython, enable edit-time type checking, including Final declaration
    from typing import Union, Final
    from collections.abc import Callable, Awaitable
    from .yocto_api import _IS_MICROPYTHON, _DYNAMIC_HELPERS
else:
    # In our micropython VM, common generic types are global built-ins
    # Others such as TypeVar should be avoided when using micropython,
    # as they produce overhead in runtime code
    # Final is translated into const() expressions before compilation
    _IS_MICROPYTHON: Final[bool] = True # noqa
    _DYNAMIC_HELPERS: Final[bool] = True # noqa

from .yocto_genericsensor_aio import YGenericSensor as YGenericSensor_aio
from .yocto_api import (
    YAPIContext, YAPI, YSensor, YMeasure
)

# --- (YGenericSensor class start)
if not _IS_MICROPYTHON:
    # For CPython, use strongly typed callback types
    try:
        YGenericSensorValueCallback = Union[Callable[['YGenericSensor', str], Awaitable[None]], None]
        YGenericSensorTimedReportCallback = Union[Callable[['YGenericSensor', YMeasure], Awaitable[None]], None]
    except TypeError:
        YGenericSensorValueCallback = Union[Callable, Awaitable]
        YGenericSensorTimedReportCallback = Union[Callable, Awaitable]

# noinspection PyProtectedMember
class YGenericSensor(YSensor):
    """
    The YGenericSensor class allows you to read and configure Yoctopuce signal
    transducers. It inherits from YSensor class the core functions to read measures,
    to register callback functions, to access the autonomous datalogger.
    This class adds the ability to configure the automatic conversion between the
    measured signal and the corresponding engineering unit.

    """
    _aio: YGenericSensor_aio
    # --- (end of YGenericSensor class start)
    if not _IS_MICROPYTHON:
        # --- (YGenericSensor return codes)
        SIGNALVALUE_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        SIGNALUNIT_INVALID: Final[str] = YAPI.INVALID_STRING
        SIGNALRANGE_INVALID: Final[str] = YAPI.INVALID_STRING
        VALUERANGE_INVALID: Final[str] = YAPI.INVALID_STRING
        SIGNALBIAS_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        SIGNALSAMPLING_HIGH_RATE: Final[int] = 0
        SIGNALSAMPLING_HIGH_RATE_FILTERED: Final[int] = 1
        SIGNALSAMPLING_LOW_NOISE: Final[int] = 2
        SIGNALSAMPLING_LOW_NOISE_FILTERED: Final[int] = 3
        SIGNALSAMPLING_HIGHEST_RATE: Final[int] = 4
        SIGNALSAMPLING_AC: Final[int] = 5
        SIGNALSAMPLING_INVALID: Final[int] = -1
        ENABLED_FALSE: Final[int] = 0
        ENABLED_TRUE: Final[int] = 1
        ENABLED_INVALID: Final[int] = -1
        # --- (end of YGenericSensor return codes)


    # --- (YGenericSensor implementation)

    @classmethod
    def FirstGenericSensor(cls) -> Union[YGenericSensor, None]:
        """
        Starts the enumeration of generic sensors currently accessible.
        Use the method YGenericSensor.nextGenericSensor() to iterate on
        next generic sensors.

        @return a pointer to a YGenericSensor object, corresponding to
                the first generic sensor currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YGenericSensor_aio.FirstGenericSensor())

    @classmethod
    def FirstGenericSensorInContext(cls, yctx: YAPIContext) -> Union[YGenericSensor, None]:
        """
        Starts the enumeration of generic sensors currently accessible.
        Use the method YGenericSensor.nextGenericSensor() to iterate on
        next generic sensors.

        @param yctx : a YAPI context.

        @return a pointer to a YGenericSensor object, corresponding to
                the first generic sensor currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YGenericSensor_aio.FirstGenericSensorInContext(yctx))

    def nextGenericSensor(self):
        """
        Continues the enumeration of generic sensors started using yFirstGenericSensor().
        Caution: You can't make any assumption about the returned generic sensors order.
        If you want to find a specific a generic sensor, use GenericSensor.findGenericSensor()
        and a hardwareID or a logical name.

        @return a pointer to a YGenericSensor object, corresponding to
                a generic sensor currently online, or a None pointer
                if there are no more generic sensors to enumerate.
        """
        return self._proxy(type(self), self._aio.nextGenericSensor())

    if not _DYNAMIC_HELPERS:
        def set_unit(self, newval: str) -> int:
            """
            Changes the measuring unit for the measured value.
            Remember to call the saveToFlash() method of the module if the
            modification must be kept.

            @param newval : a string corresponding to the measuring unit for the measured value

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_unit(newval))

    if not _DYNAMIC_HELPERS:
        def get_signalValue(self) -> float:
            """
            Returns the current value of the electrical signal measured by the sensor.

            @return a floating point number corresponding to the current value of the electrical signal
            measured by the sensor

            On failure, throws an exception or returns YGenericSensor.SIGNALVALUE_INVALID.
            """
            return self._run(self._aio.get_signalValue())

    if not _DYNAMIC_HELPERS:
        def get_signalUnit(self) -> str:
            """
            Returns the measuring unit of the electrical signal used by the sensor.

            @return a string corresponding to the measuring unit of the electrical signal used by the sensor

            On failure, throws an exception or returns YGenericSensor.SIGNALUNIT_INVALID.
            """
            return self._run(self._aio.get_signalUnit())

    if not _DYNAMIC_HELPERS:
        def get_signalRange(self) -> str:
            """
            Returns the input signal range used by the sensor.

            @return a string corresponding to the input signal range used by the sensor

            On failure, throws an exception or returns YGenericSensor.SIGNALRANGE_INVALID.
            """
            return self._run(self._aio.get_signalRange())

    if not _DYNAMIC_HELPERS:
        def set_signalRange(self, newval: str) -> int:
            """
            Changes the input signal range used by the sensor.
            When the input signal gets out of the planned range, the output value
            will be set to an arbitrary large value, whose sign indicates the direction
            of the range overrun.

            For a 4-20mA sensor, the default input signal range is "4...20".
            For a 0-10V sensor, the default input signal range is "0.1...10".
            For numeric communication interfaces, the default input signal range is
            "-999999.999...999999.999".

            Remember to call the saveToFlash()
            method of the module if the modification must be kept.

            @param newval : a string corresponding to the input signal range used by the sensor

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_signalRange(newval))

    if not _DYNAMIC_HELPERS:
        def get_valueRange(self) -> str:
            """
            Returns the physical value range measured by the sensor.

            @return a string corresponding to the physical value range measured by the sensor

            On failure, throws an exception or returns YGenericSensor.VALUERANGE_INVALID.
            """
            return self._run(self._aio.get_valueRange())

    if not _DYNAMIC_HELPERS:
        def set_valueRange(self, newval: str) -> int:
            """
            Changes the output value range, corresponding to the physical value measured
            by the sensor. The default output value range is the same as the input signal
            range (1:1 mapping), but you can change it so that the function automatically
            computes the physical value encoded by the input signal. Be aware that, as a
            side effect, the range modification may automatically modify the display resolution.

            Remember to call the saveToFlash()
            method of the module if the modification must be kept.

            @param newval : a string corresponding to the output value range, corresponding to the physical value measured
                    by the sensor

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_valueRange(newval))

    if not _DYNAMIC_HELPERS:
        def set_signalBias(self, newval: float) -> int:
            """
            Changes the electric signal bias for zero shift adjustment.
            If your electric signal reads positive when it should be zero, set up
            a positive signalBias of the same value to fix the zero shift.
            Remember to call the saveToFlash()
            method of the module if the modification must be kept.

            @param newval : a floating point number corresponding to the electric signal bias for zero shift adjustment

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_signalBias(newval))

    if not _DYNAMIC_HELPERS:
        def get_signalBias(self) -> float:
            """
            Returns the electric signal bias for zero shift adjustment.
            A positive bias means that the signal is over-reporting the measure,
            while a negative bias means that the signal is under-reporting the measure.

            @return a floating point number corresponding to the electric signal bias for zero shift adjustment

            On failure, throws an exception or returns YGenericSensor.SIGNALBIAS_INVALID.
            """
            return self._run(self._aio.get_signalBias())

    if not _DYNAMIC_HELPERS:
        def get_signalSampling(self) -> int:
            """
            Returns the electric signal sampling method to use.
            The HIGH_RATE method uses the highest sampling frequency, without any filtering.
            The HIGH_RATE_FILTERED method adds a windowed 7-sample median filter.
            The LOW_NOISE method uses a reduced acquisition frequency to reduce noise.
            The LOW_NOISE_FILTERED method combines a reduced frequency with the median filter
            to get measures as stable as possible when working on a noisy signal.

            @return a value among YGenericSensor.SIGNALSAMPLING_HIGH_RATE,
            YGenericSensor.SIGNALSAMPLING_HIGH_RATE_FILTERED, YGenericSensor.SIGNALSAMPLING_LOW_NOISE,
            YGenericSensor.SIGNALSAMPLING_LOW_NOISE_FILTERED, YGenericSensor.SIGNALSAMPLING_HIGHEST_RATE and
            YGenericSensor.SIGNALSAMPLING_AC corresponding to the electric signal sampling method to use

            On failure, throws an exception or returns YGenericSensor.SIGNALSAMPLING_INVALID.
            """
            return self._run(self._aio.get_signalSampling())

    if not _DYNAMIC_HELPERS:
        def set_signalSampling(self, newval: int) -> int:
            """
            Changes the electric signal sampling method to use.
            The HIGH_RATE method uses the highest sampling frequency, without any filtering.
            The HIGH_RATE_FILTERED method adds a windowed 7-sample median filter.
            The LOW_NOISE method uses a reduced acquisition frequency to reduce noise.
            The LOW_NOISE_FILTERED method combines a reduced frequency with the median filter
            to get measures as stable as possible when working on a noisy signal.
            Remember to call the saveToFlash()
            method of the module if the modification must be kept.

            @param newval : a value among YGenericSensor.SIGNALSAMPLING_HIGH_RATE,
            YGenericSensor.SIGNALSAMPLING_HIGH_RATE_FILTERED, YGenericSensor.SIGNALSAMPLING_LOW_NOISE,
            YGenericSensor.SIGNALSAMPLING_LOW_NOISE_FILTERED, YGenericSensor.SIGNALSAMPLING_HIGHEST_RATE and
            YGenericSensor.SIGNALSAMPLING_AC corresponding to the electric signal sampling method to use

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_signalSampling(newval))

    if not _DYNAMIC_HELPERS:
        def get_enabled(self) -> int:
            """
            Returns the activation state of this input.

            @return either YGenericSensor.ENABLED_FALSE or YGenericSensor.ENABLED_TRUE, according to the
            activation state of this input

            On failure, throws an exception or returns YGenericSensor.ENABLED_INVALID.
            """
            return self._run(self._aio.get_enabled())

    if not _DYNAMIC_HELPERS:
        def set_enabled(self, newval: int) -> int:
            """
            Changes the activation state of this input. When an input is disabled,
            its value is no more updated. On some devices, disabling an input can
            improve the refresh rate of the other active inputs.
            Remember to call the saveToFlash()
            method of the module if the modification must be kept.

            @param newval : either YGenericSensor.ENABLED_FALSE or YGenericSensor.ENABLED_TRUE, according to
            the activation state of this input

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_enabled(newval))

    @classmethod
    def FindGenericSensor(cls, func: str) -> YGenericSensor:
        """
        Retrieves a generic sensor for a given identifier.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the generic sensor is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YGenericSensor.isOnline() to test if the generic sensor is
        indeed online at a given time. In case of ambiguity when looking for
        a generic sensor by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        If a call to this object's is_online() method returns FALSE although
        you are certain that the matching device is plugged, make sure that you did
        call registerHub() at application initialization time.

        @param func : a string that uniquely characterizes the generic sensor, for instance
                RX010V01.genericSensor1.

        @return a YGenericSensor object allowing you to drive the generic sensor.
        """
        return cls._proxy(cls, YGenericSensor_aio.FindGenericSensor(func))

    @classmethod
    def FindGenericSensorInContext(cls, yctx: YAPIContext, func: str) -> YGenericSensor:
        """
        Retrieves a generic sensor for a given identifier in a YAPI context.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the generic sensor is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YGenericSensor.isOnline() to test if the generic sensor is
        indeed online at a given time. In case of ambiguity when looking for
        a generic sensor by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        @param yctx : a YAPI context
        @param func : a string that uniquely characterizes the generic sensor, for instance
                RX010V01.genericSensor1.

        @return a YGenericSensor object allowing you to drive the generic sensor.
        """
        return cls._proxy(cls, YGenericSensor_aio.FindGenericSensorInContext(yctx, func))

    if not _IS_MICROPYTHON:
        def registerValueCallback(self, callback: YGenericSensorValueCallback) -> int:
            """
            Registers the callback function that is invoked on every change of advertised value.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and the character string describing
                    the new advertised value.
            @noreturn
            """
            return super().registerValueCallback(callback)

    if not _IS_MICROPYTHON:
        def registerTimedReportCallback(self, callback: YGenericSensorTimedReportCallback) -> int:
            """
            Registers the callback function that is invoked on every periodic timed notification.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and an YMeasure object describing
                    the new advertised value.
            @noreturn
            """
            return super().registerTimedReportCallback(callback)

    if not _DYNAMIC_HELPERS:
        def zeroAdjust(self) -> int:
            """
            Adjusts the signal bias so that the current signal value is need
            precisely as zero. Remember to call the saveToFlash()
            method of the module if the modification must be kept.

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.zeroAdjust())

    # --- (end of YGenericSensor implementation)




# -*- coding: utf-8 -*-
# ********************************************************************
#
#  $Id: svn_id $
#
#  Asyncio implementation of YProximity
#
#  - - - - - - - - - License information: - - - - - - - - -
#
#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#
#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#  non-exclusive license to use, modify, copy and integrate this
#  file into your software for the sole purpose of interfacing
#  with Yoctopuce products.
#
#  You may reproduce and distribute copies of this file in
#  source or object form, as long as the sole purpose of this
#  code is to interface with Yoctopuce products. You must retain
#  this notice in the distributed source file.
#
#  You should refer to Yoctopuce General Terms and Conditions
#  for additional information regarding your rights and
#  obligations.
#
#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#  WARRANTY, OR OTHERWISE.
#
# *********************************************************************
"""
Yoctopuce library: Asyncio implementation of YProximity
version: PATCH_WITH_VERSION
requires: yocto_api_aio
"""
from __future__ import annotations

import sys

# On MicroPython, code below will be wiped out at compile time
if sys.implementation.name != "micropython":
    # In CPython, enable edit-time type checking, including Final declaration
    from typing import Any, Union, Final
    from collections.abc import Callable, Awaitable
    from .yocto_api_aio import const, _IS_MICROPYTHON
else:
    # In our micropython VM, common generic types are global built-ins
    # Others such as TypeVar should be avoided when using micropython,
    # as they produce overhead in runtime code
    # Final is translated into const() expressions before compilation
    _IS_MICROPYTHON: Final[bool] = True # noqa

from .yocto_api_aio import (
    YAPIContext, YAPI, YAPI_Exception, HwId, hwid2str, YFunction, YSensor, YMeasure
)

# --- (YProximity class start)
if not _IS_MICROPYTHON:
    # For CPython, use strongly typed callback types
    try:
        YProximityValueCallback = Union[Callable[['YProximity', str], Awaitable[None]], None]
        YProximityTimedReportCallback = Union[Callable[['YProximity', YMeasure], Awaitable[None]], None]
    except TypeError:
        YProximityValueCallback = Union[Callable, Awaitable]
        YProximityTimedReportCallback = Union[Callable, Awaitable]

# noinspection PyProtectedMember
class YProximity(YSensor):
    """
    The YProximity class allows you to read and configure Yoctopuce proximity sensors.
    It inherits from YSensor class the core functions to read measures,
    to register callback functions, and to access the autonomous datalogger.
    This class adds the ability to set up a detection threshold and to count the
    number of detected state changes.

    """
    # --- (end of YProximity class start)
    if not _IS_MICROPYTHON:
        # --- (YProximity return codes)
        SIGNALVALUE_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        DETECTIONTHRESHOLD_INVALID: Final[int] = YAPI.INVALID_UINT
        DETECTIONHYSTERESIS_INVALID: Final[int] = YAPI.INVALID_UINT
        PRESENCEMINTIME_INVALID: Final[int] = YAPI.INVALID_UINT
        REMOVALMINTIME_INVALID: Final[int] = YAPI.INVALID_UINT
        LASTTIMEAPPROACHED_INVALID: Final[int] = YAPI.INVALID_LONG
        LASTTIMEREMOVED_INVALID: Final[int] = YAPI.INVALID_LONG
        PULSECOUNTER_INVALID: Final[int] = YAPI.INVALID_LONG
        PULSETIMER_INVALID: Final[int] = YAPI.INVALID_LONG
        ISPRESENT_FALSE: Final[int] = 0
        ISPRESENT_TRUE: Final[int] = 1
        ISPRESENT_INVALID: Final[int] = -1
        PROXIMITYREPORTMODE_NUMERIC: Final[int] = 0
        PROXIMITYREPORTMODE_PRESENCE: Final[int] = 1
        PROXIMITYREPORTMODE_PULSECOUNT: Final[int] = 2
        PROXIMITYREPORTMODE_INVALID: Final[int] = -1
        # --- (end of YProximity return codes)

    # --- (YProximity attributes declaration)
    _signalValue: float
    _detectionThreshold: int
    _detectionHysteresis: int
    _presenceMinTime: int
    _removalMinTime: int
    _isPresent: int
    _lastTimeApproached: int
    _lastTimeRemoved: int
    _pulseCounter: int
    _pulseTimer: int
    _proximityReportMode: int
    _valueCallback: YProximityValueCallback
    _timedReportCallback: YProximityTimedReportCallback
    # --- (end of YProximity attributes declaration)


    def __init__(self, yctx: YAPIContext, func: str):
        super().__init__(yctx, func)
        self._className = 'Proximity'
        # --- (YProximity constructor)
        self._signalValue = YProximity.SIGNALVALUE_INVALID
        self._detectionThreshold = YProximity.DETECTIONTHRESHOLD_INVALID
        self._detectionHysteresis = YProximity.DETECTIONHYSTERESIS_INVALID
        self._presenceMinTime = YProximity.PRESENCEMINTIME_INVALID
        self._removalMinTime = YProximity.REMOVALMINTIME_INVALID
        self._isPresent = YProximity.ISPRESENT_INVALID
        self._lastTimeApproached = YProximity.LASTTIMEAPPROACHED_INVALID
        self._lastTimeRemoved = YProximity.LASTTIMEREMOVED_INVALID
        self._pulseCounter = YProximity.PULSECOUNTER_INVALID
        self._pulseTimer = YProximity.PULSETIMER_INVALID
        self._proximityReportMode = YProximity.PROXIMITYREPORTMODE_INVALID
        # --- (end of YProximity constructor)

    # --- (YProximity implementation)

    @staticmethod
    def FirstProximity() -> Union[YProximity, None]:
        """
        Starts the enumeration of proximity sensors currently accessible.
        Use the method YProximity.nextProximity() to iterate on
        next proximity sensors.

        @return a pointer to a YProximity object, corresponding to
                the first proximity sensor currently online, or a None pointer
                if there are none.
        """
        next_hwid: Union[HwId, None] = YAPI._yHash.getFirstHardwareId('Proximity')
        if not next_hwid:
            return None
        return YProximity.FindProximity(hwid2str(next_hwid))

    @staticmethod
    def FirstProximityInContext(yctx: YAPIContext) -> Union[YProximity, None]:
        """
        Starts the enumeration of proximity sensors currently accessible.
        Use the method YProximity.nextProximity() to iterate on
        next proximity sensors.

        @param yctx : a YAPI context.

        @return a pointer to a YProximity object, corresponding to
                the first proximity sensor currently online, or a None pointer
                if there are none.
        """
        next_hwid: Union[HwId, None] = yctx._yHash.getFirstHardwareId('Proximity')
        if not next_hwid:
            return None
        return YProximity.FindProximityInContext(yctx, hwid2str(next_hwid))

    def nextProximity(self):
        """
        Continues the enumeration of proximity sensors started using yFirstProximity().
        Caution: You can't make any assumption about the returned proximity sensors order.
        If you want to find a specific a proximity sensor, use Proximity.findProximity()
        and a hardwareID or a logical name.

        @return a pointer to a YProximity object, corresponding to
                a proximity sensor currently online, or a None pointer
                if there are no more proximity sensors to enumerate.
        """
        next_hwid: Union[HwId, None] = None
        try:
            hwid: HwId = self._yapi._yHash.resolveHwID(self._className, self._func)
            next_hwid = self._yapi._yHash.getNextHardwareId(self._className, hwid)
        except YAPI_Exception:
            pass
        if not next_hwid:
            return None
        return YProximity.FindProximityInContext(self._yapi, hwid2str(next_hwid))

    def _parseAttr(self, json_val: dict) -> None:
        if 'signalValue' in json_val:
            self._signalValue = round(json_val["signalValue"] / 65.536) / 1000.0
        if 'detectionThreshold' in json_val:
            self._detectionThreshold = json_val["detectionThreshold"]
        if 'detectionHysteresis' in json_val:
            self._detectionHysteresis = json_val["detectionHysteresis"]
        if 'presenceMinTime' in json_val:
            self._presenceMinTime = json_val["presenceMinTime"]
        if 'removalMinTime' in json_val:
            self._removalMinTime = json_val["removalMinTime"]
        if 'isPresent' in json_val:
            self._isPresent = json_val["isPresent"] > 0
        if 'lastTimeApproached' in json_val:
            self._lastTimeApproached = json_val["lastTimeApproached"]
        if 'lastTimeRemoved' in json_val:
            self._lastTimeRemoved = json_val["lastTimeRemoved"]
        if 'pulseCounter' in json_val:
            self._pulseCounter = json_val["pulseCounter"]
        if 'pulseTimer' in json_val:
            self._pulseTimer = json_val["pulseTimer"]
        if 'proximityReportMode' in json_val:
            self._proximityReportMode = json_val["proximityReportMode"]
        super()._parseAttr(json_val)

    async def get_signalValue(self) -> float:
        """
        Returns the current value of signal measured by the proximity sensor.

        @return a floating point number corresponding to the current value of signal measured by the proximity sensor

        On failure, throws an exception or returns YProximity.SIGNALVALUE_INVALID.
        """
        res: float
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YProximity.SIGNALVALUE_INVALID
        res = round(self._signalValue * 1000) / 1000
        return res

    async def get_detectionThreshold(self) -> int:
        """
        Returns the threshold used to determine the logical state of the proximity sensor, when considered
        as a binary input (on/off).

        @return an integer corresponding to the threshold used to determine the logical state of the
        proximity sensor, when considered
                as a binary input (on/off)

        On failure, throws an exception or returns YProximity.DETECTIONTHRESHOLD_INVALID.
        """
        res: int
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YProximity.DETECTIONTHRESHOLD_INVALID
        res = self._detectionThreshold
        return res

    async def set_detectionThreshold(self, newval: int) -> int:
        """
        Changes the threshold used to determine the logical state of the proximity sensor, when considered
        as a binary input (on/off).
        Remember to call the saveToFlash() method of the module if the modification must be kept.

        @param newval : an integer corresponding to the threshold used to determine the logical state of
        the proximity sensor, when considered
                as a binary input (on/off)

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(newval)
        return await self._setAttr("detectionThreshold", rest_val)

    async def get_detectionHysteresis(self) -> int:
        """
        Returns the hysteresis used to determine the logical state of the proximity sensor, when considered
        as a binary input (on/off).

        @return an integer corresponding to the hysteresis used to determine the logical state of the
        proximity sensor, when considered
                as a binary input (on/off)

        On failure, throws an exception or returns YProximity.DETECTIONHYSTERESIS_INVALID.
        """
        res: int
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YProximity.DETECTIONHYSTERESIS_INVALID
        res = self._detectionHysteresis
        return res

    async def set_detectionHysteresis(self, newval: int) -> int:
        """
        Changes the hysteresis used to determine the logical state of the proximity sensor, when considered
        as a binary input (on/off).
        Remember to call the saveToFlash() method of the module if the modification must be kept.

        @param newval : an integer corresponding to the hysteresis used to determine the logical state of
        the proximity sensor, when considered
                as a binary input (on/off)

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(newval)
        return await self._setAttr("detectionHysteresis", rest_val)

    async def get_presenceMinTime(self) -> int:
        """
        Returns the minimal detection duration before signalling a presence event. Any shorter detection is
        considered as noise or bounce (false positive) and filtered out.

        @return an integer corresponding to the minimal detection duration before signalling a presence event

        On failure, throws an exception or returns YProximity.PRESENCEMINTIME_INVALID.
        """
        res: int
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YProximity.PRESENCEMINTIME_INVALID
        res = self._presenceMinTime
        return res

    async def set_presenceMinTime(self, newval: int) -> int:
        """
        Changes the minimal detection duration before signalling a presence event. Any shorter detection is
        considered as noise or bounce (false positive) and filtered out.
        Remember to call the saveToFlash() method of the module if the modification must be kept.

        @param newval : an integer corresponding to the minimal detection duration before signalling a presence event

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(newval)
        return await self._setAttr("presenceMinTime", rest_val)

    async def get_removalMinTime(self) -> int:
        """
        Returns the minimal detection duration before signalling a removal event. Any shorter detection is
        considered as noise or bounce (false positive) and filtered out.

        @return an integer corresponding to the minimal detection duration before signalling a removal event

        On failure, throws an exception or returns YProximity.REMOVALMINTIME_INVALID.
        """
        res: int
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YProximity.REMOVALMINTIME_INVALID
        res = self._removalMinTime
        return res

    async def set_removalMinTime(self, newval: int) -> int:
        """
        Changes the minimal detection duration before signalling a removal event. Any shorter detection is
        considered as noise or bounce (false positive) and filtered out.
        Remember to call the saveToFlash() method of the module if the modification must be kept.

        @param newval : an integer corresponding to the minimal detection duration before signalling a removal event

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(newval)
        return await self._setAttr("removalMinTime", rest_val)

    async def get_isPresent(self) -> int:
        """
        Returns true if the input (considered as binary) is active (detection value is smaller than the
        specified threshold), and false otherwise.

        @return either YProximity.ISPRESENT_FALSE or YProximity.ISPRESENT_TRUE, according to true if the
        input (considered as binary) is active (detection value is smaller than the specified threshold),
        and false otherwise

        On failure, throws an exception or returns YProximity.ISPRESENT_INVALID.
        """
        res: int
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YProximity.ISPRESENT_INVALID
        res = self._isPresent
        return res

    async def get_lastTimeApproached(self) -> int:
        """
        Returns the number of elapsed milliseconds between the module power on and the last observed
        detection (the input contact transitioned from absent to present).

        @return an integer corresponding to the number of elapsed milliseconds between the module power on
        and the last observed
                detection (the input contact transitioned from absent to present)

        On failure, throws an exception or returns YProximity.LASTTIMEAPPROACHED_INVALID.
        """
        res: int
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YProximity.LASTTIMEAPPROACHED_INVALID
        res = self._lastTimeApproached
        return res

    async def get_lastTimeRemoved(self) -> int:
        """
        Returns the number of elapsed milliseconds between the module power on and the last observed
        detection (the input contact transitioned from present to absent).

        @return an integer corresponding to the number of elapsed milliseconds between the module power on
        and the last observed
                detection (the input contact transitioned from present to absent)

        On failure, throws an exception or returns YProximity.LASTTIMEREMOVED_INVALID.
        """
        res: int
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YProximity.LASTTIMEREMOVED_INVALID
        res = self._lastTimeRemoved
        return res

    async def get_pulseCounter(self) -> int:
        """
        Returns the pulse counter value. The value is a 32 bit integer. In case
        of overflow (>=2^32), the counter will wrap. To reset the counter, just
        call the resetCounter() method.

        @return an integer corresponding to the pulse counter value

        On failure, throws an exception or returns YProximity.PULSECOUNTER_INVALID.
        """
        res: int
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YProximity.PULSECOUNTER_INVALID
        res = self._pulseCounter
        return res

    async def set_pulseCounter(self, newval: int) -> int:
        rest_val = str(newval)
        return await self._setAttr("pulseCounter", rest_val)

    async def get_pulseTimer(self) -> int:
        """
        Returns the timer of the pulse counter (ms).

        @return an integer corresponding to the timer of the pulse counter (ms)

        On failure, throws an exception or returns YProximity.PULSETIMER_INVALID.
        """
        res: int
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YProximity.PULSETIMER_INVALID
        res = self._pulseTimer
        return res

    async def get_proximityReportMode(self) -> int:
        """
        Returns the parameter (sensor value, presence or pulse count) returned by the get_currentValue
        function and callbacks.

        @return a value among YProximity.PROXIMITYREPORTMODE_NUMERIC,
        YProximity.PROXIMITYREPORTMODE_PRESENCE and YProximity.PROXIMITYREPORTMODE_PULSECOUNT corresponding
        to the parameter (sensor value, presence or pulse count) returned by the get_currentValue function and callbacks

        On failure, throws an exception or returns YProximity.PROXIMITYREPORTMODE_INVALID.
        """
        res: int
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YProximity.PROXIMITYREPORTMODE_INVALID
        res = self._proximityReportMode
        return res

    async def set_proximityReportMode(self, newval: int) -> int:
        """
        Changes the  parameter  type (sensor value, presence or pulse count) returned by the
        get_currentValue function and callbacks.
        The edge count value is limited to the 6 lowest digits. For values greater than one million, use
        get_pulseCounter().
        Remember to call the saveToFlash() method of the module if the modification must be kept.

        @param newval : a value among YProximity.PROXIMITYREPORTMODE_NUMERIC,
        YProximity.PROXIMITYREPORTMODE_PRESENCE and YProximity.PROXIMITYREPORTMODE_PULSECOUNT corresponding
        to the  parameter  type (sensor value, presence or pulse count) returned by the get_currentValue
        function and callbacks

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(newval)
        return await self._setAttr("proximityReportMode", rest_val)

    @staticmethod
    def FindProximity(func: str) -> YProximity:
        """
        Retrieves a proximity sensor for a given identifier.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the proximity sensor is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YProximity.isOnline() to test if the proximity sensor is
        indeed online at a given time. In case of ambiguity when looking for
        a proximity sensor by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        If a call to this object's is_online() method returns FALSE although
        you are certain that the matching device is plugged, make sure that you did
        call registerHub() at application initialization time.

        @param func : a string that uniquely characterizes the proximity sensor, for instance
                YPROXIM1.proximity1.

        @return a YProximity object allowing you to drive the proximity sensor.
        """
        obj: Union[YProximity, None]
        obj = YFunction._FindFromCache("Proximity", func)
        if obj is None:
            obj = YProximity(YAPI, func)
            YFunction._AddToCache("Proximity", func, obj)
        return obj

    @staticmethod
    def FindProximityInContext(yctx: YAPIContext, func: str) -> YProximity:
        """
        Retrieves a proximity sensor for a given identifier in a YAPI context.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the proximity sensor is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YProximity.isOnline() to test if the proximity sensor is
        indeed online at a given time. In case of ambiguity when looking for
        a proximity sensor by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        @param yctx : a YAPI context
        @param func : a string that uniquely characterizes the proximity sensor, for instance
                YPROXIM1.proximity1.

        @return a YProximity object allowing you to drive the proximity sensor.
        """
        obj: Union[YProximity, None]
        obj = YFunction._FindFromCacheInContext(yctx, "Proximity", func)
        if obj is None:
            obj = YProximity(yctx, func)
            YFunction._AddToCache("Proximity", func, obj)
        return obj

    if not _IS_MICROPYTHON:
        async def registerValueCallback(self, callback: YProximityValueCallback) -> int:
            """
            Registers the callback function that is invoked on every change of advertised value.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and the character string describing
                    the new advertised value.
            @noreturn
            """
            return await super().registerValueCallback(callback)

    if not _IS_MICROPYTHON:
        async def registerTimedReportCallback(self, callback: YProximityTimedReportCallback) -> int:
            """
            Registers the callback function that is invoked on every periodic timed notification.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and an YMeasure object describing
                    the new advertised value.
            @noreturn
            """
            return await super().registerTimedReportCallback(callback)

    async def resetCounter(self) -> int:
        """
        Resets the pulse counter value as well as its timer.

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        return await self.set_pulseCounter(0)

    # --- (end of YProximity implementation)

